@@ -190,7 +190,7 @@ log_queue_fifo_drop_messages_from_input_queue(LogQueueFifo *self, InputQueue *in
 
       iv_list_del(&node->list);
       input_queue->len--;
-      stats_counter_inc(self->super.dropped_messages);
+      log_queue_dropped_messages_inc(&self->super);
       log_msg_free_queue_node(node);
 
       LogMessage *msg = node->msg;
@@ -394,7 +394,7 @@ log_queue_fifo_push_tail(LogQueue *s, LogMessage *msg, const LogPathOptions *pat
 
   if (_message_has_to_be_dropped(self, path_options))
     {
-      stats_counter_inc(self->super.dropped_messages);
+      log_queue_dropped_messages_inc(&self->super);
       g_mutex_unlock(&self->super.lock);
 
       _drop_message(msg, path_options);
@@ -655,14 +655,18 @@ log_queue_fifo_free(LogQueue *s)
 }
 
 LogQueue *
-log_queue_fifo_new(gint log_fifo_size, const gchar *persist_name)
+log_queue_fifo_new(gint log_fifo_size, const gchar *persist_name, gint stats_level,
+                   const StatsClusterKeyBuilder *driver_sck_builder, StatsClusterKeyBuilder *queue_sck_builder)
 {
   LogQueueFifo *self;
 
   gint max_threads = main_loop_worker_get_max_number_of_threads();
   self = g_malloc0(sizeof(LogQueueFifo) + max_threads * sizeof(self->input_queues[0]));
 
-  log_queue_init_instance(&self->super, persist_name);
+  if (queue_sck_builder)
+    stats_cluster_key_builder_set_name_prefix(queue_sck_builder, "memory_queue_");
+
+  log_queue_init_instance(&self->super, persist_name, stats_level, driver_sck_builder, queue_sck_builder);
   self->super.type = log_queue_fifo_type;
   self->super.use_backlog = FALSE;
   self->super.get_length = log_queue_fifo_get_length;
@@ -694,9 +698,11 @@ log_queue_fifo_new(gint log_fifo_size, const gchar *persist_name)
 }
 
 LogQueue *
-log_queue_fifo_legacy_new(gint log_fifo_size, const gchar *persist_name)
+log_queue_fifo_legacy_new(gint log_fifo_size, const gchar *persist_name, gint stats_level,
+                          const StatsClusterKeyBuilder *driver_sck_builder, StatsClusterKeyBuilder *queue_sck_builder)
 {
-  LogQueueFifo *self = (LogQueueFifo *) log_queue_fifo_new(log_fifo_size, persist_name);
+  LogQueueFifo *self = (LogQueueFifo *) log_queue_fifo_new(log_fifo_size, persist_name, stats_level,
+                                                           driver_sck_builder, queue_sck_builder);
   self->use_legacy_fifo_size = TRUE;
   return &self->super;
 }
