@@ -354,7 +354,7 @@ log_reader_update_watches(LogReader *self)
  *****************************************************************************/
 
 static void
-log_reader_work_perform(void *s, GIOCondition cond)
+log_reader_work_perform(void *s, gpointer arg)
 {
   LogReader *self = (LogReader *) s;
 
@@ -362,7 +362,7 @@ log_reader_work_perform(void *s, GIOCondition cond)
 }
 
 static void
-log_reader_work_finished(void *s)
+log_reader_work_finished(void *s, gpointer arg)
 {
   LogReader *self = (LogReader *) s;
 
@@ -457,10 +457,20 @@ log_reader_handle_line(LogReader *self, const guchar *line, gint length, LogTran
   msg_format_parse_into(&self->options->parse_options, m, line, length);
 
   _log_reader_insert_msg_length_stats(self, length);
+
+  log_msg_set_recvd_rawmsg_size(m, length);
+
   if (aux)
     {
       log_msg_set_saddr(m, aux->peer_addr ? : self->peer_addr);
       log_msg_set_daddr(m, aux->local_addr ? : self->local_addr);
+      if (aux->timestamp.tv_sec)
+        {
+          /* accurate timestamp was received from the transport layer, use
+           * that instead of the one we generated */
+          m->timestamps[LM_TS_RECVD].ut_sec = aux->timestamp.tv_sec;
+          m->timestamps[LM_TS_RECVD].ut_usec = aux->timestamp.tv_nsec / 1000;
+        }
       m->proto = aux->proto;
     }
   log_msg_refcache_start_producer(m);
@@ -559,7 +569,7 @@ log_reader_io_handle_in(gpointer s)
   log_reader_disable_watches(self);
   if ((self->options->flags & LR_THREADED))
     {
-      main_loop_io_worker_job_submit(&self->io_job, G_IO_IN);
+      main_loop_io_worker_job_submit(&self->io_job, NULL);
     }
   else
     {
@@ -574,8 +584,8 @@ log_reader_io_handle_in(gpointer s)
       if (!main_loop_worker_job_quit())
         {
           log_pipe_ref(&self->super.super);
-          log_reader_work_perform(s, G_IO_IN);
-          log_reader_work_finished(s);
+          log_reader_work_perform(s, NULL);
+          log_reader_work_finished(s, NULL);
           log_pipe_unref(&self->super.super);
         }
     }
@@ -584,10 +594,12 @@ log_reader_io_handle_in(gpointer s)
 static void
 _register_aggregated_stats(LogReader *self)
 {
+  gint level = log_pipe_is_internal(&self->super.super) ? STATS_LEVEL3 : self->super.options->stats_level;
+
   StatsClusterKey sc_key_eps_input;
-  stats_cluster_logpipe_key_legacy_set(&sc_key_eps_input, self->super.options->stats_source | SCS_SOURCE,
-                                       self->super.stats_id,
-                                       self->super.stats_instance);
+  stats_cluster_single_key_legacy_set_with_name(&sc_key_eps_input, self->super.options->stats_source | SCS_SOURCE,
+                                                self->super.stats_id,
+                                                self->super.stats_instance, "processed");
 
   stats_aggregator_lock();
   StatsClusterKey sc_key;
@@ -595,18 +607,18 @@ _register_aggregated_stats(LogReader *self)
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE,
                                                 self->super.stats_id,
                                                 self->super.stats_instance, "msg_size_max");
-  stats_register_aggregator_maximum(self->super.options->stats_level, &sc_key, &self->max_message_size);
+  stats_register_aggregator_maximum(level, &sc_key, &self->max_message_size);
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE,
                                                 self->super.stats_id,
                                                 self->super.stats_instance, "msg_size_avg");
-  stats_register_aggregator_average(self->super.options->stats_level, &sc_key, &self->average_messages_size);
+  stats_register_aggregator_average(level, &sc_key, &self->average_messages_size);
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->super.options->stats_source | SCS_SOURCE,
                                                 self->super.stats_id,
                                                 self->super.stats_instance, "eps");
-  stats_register_aggregator_cps(self->super.options->stats_level, &sc_key, &sc_key_eps_input, SC_TYPE_PROCESSED,
-                                &self->CPS);
+
+  stats_register_aggregator_cps(level, &sc_key, &sc_key_eps_input, SC_TYPE_SINGLE_VALUE, &self->CPS);
 
   stats_aggregator_unlock();
 }
@@ -695,10 +707,10 @@ log_reader_init_watches(LogReader *self)
 
   main_loop_io_worker_job_init(&self->io_job);
   self->io_job.user_data = self;
-  self->io_job.work = (void (*)(void *, GIOCondition)) log_reader_work_perform;
-  self->io_job.completion = (void (*)(void *)) log_reader_work_finished;
-  self->io_job.engage = (void (*)(void *)) log_pipe_ref;
-  self->io_job.release = (void (*)(void *)) log_pipe_unref;
+  self->io_job.work = log_reader_work_perform;
+  self->io_job.completion = log_reader_work_finished;
+  self->io_job.engage = (void (*)(gpointer)) log_pipe_ref;
+  self->io_job.release = (void (*)(gpointer)) log_pipe_unref;
 }
 
 static void
@@ -750,6 +762,7 @@ log_reader_new(GlobalConfig *cfg)
   self->super.super.free_fn = log_reader_free;
   self->super.wakeup = log_reader_wakeup;
   self->super.schedule_dynamic_window_realloc = _schedule_dynamic_window_realloc;
+  self->super.metrics.raw_bytes_enabled = TRUE;
   self->immediate_check = FALSE;
   log_reader_init_watches(self);
   g_mutex_init(&self->pending_close_lock);
