@@ -26,6 +26,7 @@
 #include "messages.h"
 #include "host-resolve.h"
 #include "stats/stats-registry.h"
+#include "stats/stats-cluster-logpipe.h"
 #include "stats/stats-cluster-single.h"
 #include "msg-stats.h"
 #include "logmsg/tags.h"
@@ -75,7 +76,7 @@ _flow_control_window_size_adjust(LogSource *self, guint32 window_size_increment,
     window_size_increment = _take_reclaimed_window(self, window_size_increment);
 
   gsize old_window_size = window_size_counter_add(&self->window_size, window_size_increment, &suspended);
-  stats_counter_add(self->stat_window_size, window_size_increment);
+  stats_counter_add(self->metrics.stat_window_size, window_size_increment);
 
   msg_diagnostics("Window size adjustment",
                   evt_tag_int("old_window_size", old_window_size),
@@ -233,10 +234,10 @@ _release_dynamic_window(LogSource *self)
             log_pipe_location_tag(&self->super));
 
   self->full_window_size -= dynamic_part;
-  stats_counter_sub(self->stat_full_window, dynamic_part);
+  stats_counter_sub(self->metrics.stat_full_window, dynamic_part);
 
   window_size_counter_sub(&self->window_size, dynamic_part, NULL);
-  stats_counter_sub(self->stat_window_size, dynamic_part);
+  stats_counter_sub(self->metrics.stat_window_size, dynamic_part);
   dynamic_window_release(&self->dynamic_window, dynamic_part);
 
   dynamic_window_pool_unref(self->dynamic_window.pool);
@@ -254,10 +255,10 @@ _inc_balanced(LogSource *self, gsize inc)
             evt_tag_int("new_full_window_size", self->full_window_size + offered_dynamic));
 
   self->full_window_size += offered_dynamic;
-  stats_counter_add(self->stat_full_window, offered_dynamic);
+  stats_counter_add(self->metrics.stat_full_window, offered_dynamic);
 
   gsize old_window_size = window_size_counter_add(&self->window_size, offered_dynamic, NULL);
-  stats_counter_add(self->stat_window_size, offered_dynamic);
+  stats_counter_add(self->metrics.stat_window_size, offered_dynamic);
   if (old_window_size == 0 && offered_dynamic != 0)
     log_source_wakeup(self);
 }
@@ -287,7 +288,7 @@ _dec_balanced(LogSource *self, gsize dec)
     }
 
   window_size_counter_sub(&self->window_size, dec, NULL);
-  stats_counter_sub(self->stat_window_size, dec);
+  stats_counter_sub(self->metrics.stat_window_size, dec);
 
   msg_trace("Balance::decrease",
             log_pipe_location_tag(&self->super),
@@ -297,7 +298,7 @@ _dec_balanced(LogSource *self, gsize dec)
             evt_tag_int("to_be_reclaimed", remaining_sub));
 
   self->full_window_size = new_full_window_size;
-  stats_counter_set(self->stat_full_window, new_full_window_size);
+  stats_counter_set(self->metrics.stat_full_window, new_full_window_size);
   dynamic_window_release(&self->dynamic_window, dec);
 }
 
@@ -312,7 +313,7 @@ _reclaim_window_instead_of_rebalance(LogSource *self)
   if (total_reclaim > 0)
     {
       self->full_window_size -= total_reclaim;
-      stats_counter_sub(self->stat_full_window, total_reclaim);
+      stats_counter_sub(self->metrics.stat_full_window, total_reclaim);
       dynamic_window_release(&self->dynamic_window, total_reclaim);
     }
   else
@@ -420,16 +421,16 @@ _register_window_stats(LogSource *self)
   StatsClusterKey sc_key;
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
                                                 instance_name, "free_window");
-  self->stat_window_size_cluster = stats_register_dynamic_counter(4, &sc_key, SC_TYPE_SINGLE_VALUE,
-                                   &self->stat_window_size);
-  stats_counter_set(self->stat_window_size, window_size_counter_get(&self->window_size, NULL));
+  self->metrics.stat_window_size_cluster = stats_register_dynamic_counter(4, &sc_key, SC_TYPE_SINGLE_VALUE,
+                                           &self->metrics.stat_window_size);
+  stats_counter_set(self->metrics.stat_window_size, window_size_counter_get(&self->window_size, NULL));
 
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
                                                 instance_name, "full_window");
-  self->stat_full_window_cluster = stats_register_dynamic_counter(4, &sc_key, SC_TYPE_SINGLE_VALUE,
-                                   &self->stat_full_window);
-  stats_counter_set(self->stat_full_window, self->full_window_size);
+  self->metrics.stat_full_window_cluster = stats_register_dynamic_counter(4, &sc_key, SC_TYPE_SINGLE_VALUE,
+                                           &self->metrics.stat_full_window);
+  stats_counter_set(self->metrics.stat_full_window, self->full_window_size);
 
 }
 
@@ -439,8 +440,10 @@ _unregister_window_stats(LogSource *self)
   if (!stats_check_level(4))
     return;
 
-  stats_unregister_dynamic_counter(self->stat_window_size_cluster, SC_TYPE_SINGLE_VALUE, &self->stat_window_size);
-  stats_unregister_dynamic_counter(self->stat_full_window_cluster, SC_TYPE_SINGLE_VALUE, &self->stat_full_window);
+  stats_unregister_dynamic_counter(self->metrics.stat_window_size_cluster, SC_TYPE_SINGLE_VALUE,
+                                   &self->metrics.stat_window_size);
+  stats_unregister_dynamic_counter(self->metrics.stat_full_window_cluster, SC_TYPE_SINGLE_VALUE,
+                                   &self->metrics.stat_full_window);
 }
 
 static inline void
@@ -454,6 +457,67 @@ _create_ack_tracker_if_not_exists(LogSource *self)
     }
 }
 
+static void
+_register_raw_bytes_stats(LogSource *self, gint stats_level)
+{
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->stats_id),
+    stats_cluster_label("driver_instance", self->stats_instance),
+  };
+
+  StatsClusterKey input_bytes_key;
+  stats_cluster_single_key_set(&input_bytes_key, "input_event_bytes_total", labels, G_N_ELEMENTS(labels));
+  stats_byte_counter_init(&self->metrics.recvd_bytes, &input_bytes_key, stats_level, SBCP_KIB);
+}
+
+static void
+_unregister_raw_bytes_stats(LogSource *self)
+{
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->stats_id),
+    stats_cluster_label("driver_instance", self->stats_instance),
+  };
+
+  StatsClusterKey input_bytes_key;
+  stats_cluster_single_key_set(&input_bytes_key, "input_event_bytes_total", labels, G_N_ELEMENTS(labels));
+  stats_byte_counter_deinit(&self->metrics.recvd_bytes, &input_bytes_key);
+}
+
+static void
+_register_counters(LogSource *self)
+{
+  stats_lock();
+  StatsClusterKey sc_key;
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->stats_id),
+    stats_cluster_label("driver_instance", self->stats_instance),
+  };
+
+  gint level = log_pipe_is_internal(&self->super) ? STATS_LEVEL3 : self->options->stats_level;
+
+  stats_cluster_single_key_set(&sc_key, "input_events_total", labels, G_N_ELEMENTS(labels));
+  stats_cluster_single_key_add_legacy_alias_with_name(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
+                                                      self->stats_instance, "processed");
+  stats_register_counter(level, &sc_key, SC_TYPE_SINGLE_VALUE, &self->metrics.recvd_messages);
+
+  stats_cluster_logpipe_key_legacy_set(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
+                                       self->stats_instance);
+  stats_register_counter(level, &sc_key, SC_TYPE_STAMP, &self->metrics.last_message_seen);
+
+  _register_window_stats(self);
+
+  stats_unlock();
+
+  if (self->metrics.raw_bytes_enabled)
+    {
+      level = log_pipe_is_internal(&self->super) ? STATS_LEVEL3 : STATS_LEVEL1;
+      _register_raw_bytes_stats(self, level);
+    }
+}
+
 gboolean
 log_source_init(LogPipe *s)
 {
@@ -466,6 +530,17 @@ log_source_init(LogPipe *s)
       return FALSE;
     }
 
+  _register_counters(self);
+
+  return TRUE;
+}
+
+static void
+_unregister_counters(LogSource *self)
+{
+  if (self->metrics.raw_bytes_enabled)
+    _unregister_raw_bytes_stats(self);
+
   stats_lock();
   StatsClusterKey sc_key;
   StatsClusterLabel labels[] =
@@ -473,18 +548,18 @@ log_source_init(LogPipe *s)
     stats_cluster_label("id", self->stats_id),
     stats_cluster_label("driver_instance", self->stats_instance),
   };
-  stats_cluster_logpipe_key_set(&sc_key, "input_events_total", labels, G_N_ELEMENTS(labels));
-  stats_cluster_logpipe_key_add_legacy_alias(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
-                                             self->stats_instance);
-  stats_register_counter(self->options->stats_level, &sc_key,
-                         SC_TYPE_PROCESSED, &self->recvd_messages);
-  stats_register_counter(self->options->stats_level, &sc_key, SC_TYPE_STAMP, &self->last_message_seen);
+  stats_cluster_single_key_set(&sc_key, "input_events_total", labels, G_N_ELEMENTS(labels));
+  stats_cluster_single_key_add_legacy_alias_with_name(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
+                                                      self->stats_instance, "processed");
+  stats_unregister_counter(&sc_key, SC_TYPE_SINGLE_VALUE, &self->metrics.recvd_messages);
 
-  _register_window_stats(self);
+  stats_cluster_logpipe_key_legacy_set(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
+                                       self->stats_instance);
+  stats_unregister_counter(&sc_key, SC_TYPE_STAMP, &self->metrics.last_message_seen);
+
+  _unregister_window_stats(self);
 
   stats_unlock();
-
-  return TRUE;
 }
 
 gboolean
@@ -493,22 +568,7 @@ log_source_deinit(LogPipe *s)
   LogSource *self = (LogSource *) s;
   ack_tracker_deinit(self->ack_tracker);
 
-  stats_lock();
-  StatsClusterKey sc_key;
-  StatsClusterLabel labels[] =
-  {
-    stats_cluster_label("id", self->stats_id),
-    stats_cluster_label("driver_instance", self->stats_instance),
-  };
-  stats_cluster_logpipe_key_set(&sc_key, "input_events_total", labels, G_N_ELEMENTS(labels));
-  stats_cluster_logpipe_key_add_legacy_alias(&sc_key, self->options->stats_source | SCS_SOURCE, self->stats_id,
-                                             self->stats_instance);
-  stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED, &self->recvd_messages);
-  stats_unregister_counter(&sc_key, SC_TYPE_STAMP, &self->last_message_seen);
-
-  _unregister_window_stats(self);
-
-  stats_unlock();
+  _unregister_counters(self);
 
   return TRUE;
 }
@@ -528,7 +588,7 @@ log_source_post(LogSource *self, LogMessage *msg)
   msg->ack_func = log_source_msg_ack;
 
   old_window_size = window_size_counter_sub(&self->window_size, 1, NULL);
-  stats_counter_sub(self->stat_window_size, 1);
+  stats_counter_sub(self->metrics.stat_window_size, 1);
 
   if (G_UNLIKELY(old_window_size == 1))
     {
@@ -653,8 +713,9 @@ log_source_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options
 
   /* message setup finished, send it out */
 
-  stats_counter_inc(self->recvd_messages);
-  stats_counter_set(self->last_message_seen, msg->timestamps[LM_TS_RECVD].ut_sec);
+  stats_counter_inc(self->metrics.recvd_messages);
+  stats_counter_set(self->metrics.last_message_seen, msg->timestamps[LM_TS_RECVD].ut_sec);
+  stats_byte_counter_add(&self->metrics.recvd_bytes, msg->recvd_rawmsg_size);
   log_pipe_forward_msg(s, msg, path_options);
 
   if (accurate_nanosleep && self->threaded && self->window_full_sleep_nsec > 0 && !log_source_free_to_send(self))
