@@ -34,9 +34,6 @@
 #define MAX_RETRIES_ON_ERROR_DEFAULT 3
 #define MAX_RETRIES_BEFORE_SUSPEND_DEFAULT 3
 
-static void _init_stats_legacy_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key);
-static void _init_stats_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key);
-
 const gchar *
 log_threaded_result_to_str(LogThreadedResult self)
 {
@@ -86,7 +83,7 @@ void
 log_threaded_dest_worker_ack_messages(LogThreadedDestWorker *self, gint batch_size)
 {
   log_queue_ack_backlog(self->queue, batch_size);
-  stats_counter_add(self->owner->written_messages, batch_size);
+  stats_counter_add(self->owner->metrics.written_messages, batch_size);
   self->retries_on_error_counter = 0;
   self->batch_size -= batch_size;
 }
@@ -95,7 +92,7 @@ void
 log_threaded_dest_worker_drop_messages(LogThreadedDestWorker *self, gint batch_size)
 {
   log_queue_ack_backlog(self->queue, batch_size);
-  stats_counter_add(self->owner->dropped_messages, batch_size);
+  stats_counter_add(self->owner->metrics.dropped_messages, batch_size);
   self->retries_on_error_counter = 0;
   self->batch_size -= batch_size;
 }
@@ -656,28 +653,6 @@ _init_watches(LogThreadedDestWorker *self)
   self->do_work.handler = _perform_work;
 }
 
-static void
-_register_worker_stats(LogThreadedDestWorker *self)
-{
-  StatsClusterKey sc_key;
-
-  stats_lock();
-  _init_stats_key(self->owner, &sc_key);
-  log_queue_register_stats_counters(self->queue, 0, &sc_key);
-  stats_unlock();
-}
-
-static void
-_unregister_worker_stats(LogThreadedDestWorker *self)
-{
-  StatsClusterKey sc_key;
-
-  stats_lock();
-  _init_stats_key(self->owner, &sc_key);
-  log_queue_unregister_stats_counters(self->queue, &sc_key);
-  stats_unlock();
-}
-
 static void
 _perform_final_flush(LogThreadedDestWorker *self)
 {
@@ -768,11 +743,29 @@ log_threaded_dest_worker_start(LogThreadedDestWorker *self)
   return main_loop_threaded_worker_start(&self->thread);
 }
 
+static void
+_init_queue_sck_builder(LogThreadedDestWorker *self, StatsClusterKeyBuilder *builder)
+{
+  stats_cluster_key_builder_add_label(builder, stats_cluster_label("id", self->owner->super.super.id ? : ""));
+  stats_cluster_key_builder_add_label(builder, stats_cluster_label("driver_instance",
+                                      self->owner->format_stats_instance(self->owner)));
+
+  gchar worker_index_str[8];
+  g_snprintf(worker_index_str, sizeof(worker_index_str), "%d", self->worker_index);
+  stats_cluster_key_builder_add_label(builder, stats_cluster_label("worker", worker_index_str));
+}
+
 static gboolean
-_acquire_worker_queue(LogThreadedDestWorker *self)
+_acquire_worker_queue(LogThreadedDestWorker *self, gint stats_level, const StatsClusterKeyBuilder *driver_sck_builder)
 {
   gchar *persist_name = _format_queue_persist_name(self);
-  self->queue = log_dest_driver_acquire_queue(&self->owner->super, persist_name);
+  StatsClusterKeyBuilder *queue_sck_builder = stats_cluster_key_builder_new();
+  _init_queue_sck_builder(self, queue_sck_builder);
+
+  self->queue = log_dest_driver_acquire_queue(&self->owner->super, persist_name, stats_level, driver_sck_builder,
+                                              queue_sck_builder);
+
+  stats_cluster_key_builder_free(queue_sck_builder);
   g_free(persist_name);
 
   if (!self->queue)
@@ -783,26 +776,56 @@ _acquire_worker_queue(LogThreadedDestWorker *self)
   return TRUE;
 }
 
+static void
+_register_raw_bytes_stats(LogThreadedDestWorker *self)
+{
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->owner->super.super.id ? : ""),
+    stats_cluster_label("driver_instance", self->owner->format_stats_instance(self->owner)),
+  };
+
+  gint level = log_pipe_is_internal(&self->owner->super.super.super) ? STATS_LEVEL3 : STATS_LEVEL1;
+
+  StatsClusterKey output_bytes_key;
+  stats_cluster_single_key_set(&output_bytes_key, "output_event_bytes_total", labels, G_N_ELEMENTS(labels));
+  stats_byte_counter_init(&self->metrics.written_bytes, &output_bytes_key, level, SBCP_KIB);
+}
+
+static void
+_unregister_raw_bytes_stats(LogThreadedDestWorker *self)
+{
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->owner->super.super.id ? : ""),
+    stats_cluster_label("driver_instance", self->owner->format_stats_instance(self->owner)),
+  };
+
+  StatsClusterKey output_bytes_key;
+  stats_cluster_single_key_set(&output_bytes_key, "output_event_bytes_total", labels, G_N_ELEMENTS(labels));
+  stats_byte_counter_deinit(&self->metrics.written_bytes, &output_bytes_key);
+}
+
 gboolean
 log_threaded_dest_worker_init_method(LogThreadedDestWorker *self)
 {
   if (self->time_reopen == -1)
     self->time_reopen = self->owner->time_reopen;
 
-  _register_worker_stats(self);
-
   return TRUE;
 }
 
 void
 log_threaded_dest_worker_deinit_method(LogThreadedDestWorker *self)
 {
-  _unregister_worker_stats(self);
 }
 
 void
 log_threaded_dest_worker_free_method(LogThreadedDestWorker *self)
 {
+  if (self->owner->metrics.raw_bytes_enabled)
+    _unregister_raw_bytes_stats(self);
+
   main_loop_threaded_worker_clear(&self->thread);
 }
 
@@ -821,6 +844,10 @@ log_threaded_dest_worker_init_instance(LogThreadedDestWorker *self, LogThreadedD
   self->owner = owner;
   self->time_reopen = -1;
   _init_watches(self);
+
+  /* cannot be moved to the thread's init() as neither StatsByteCounter nor format_stats_instance() is thread-safe */
+  if (self->owner->metrics.raw_bytes_enabled)
+    _register_raw_bytes_stats(self);
 }
 
 void
@@ -961,7 +988,7 @@ log_threaded_dest_driver_queue(LogPipe *s, LogMessage *msg,
   log_msg_add_ack(msg, path_options);
   log_queue_push_tail(dw->queue, log_msg_ref(msg), path_options);
 
-  stats_counter_inc(self->processed_messages);
+  stats_counter_inc(self->metrics.processed_messages);
 
   log_dest_driver_queue_method(s, msg, path_options);
 }
@@ -974,37 +1001,31 @@ _init_stats_legacy_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key)
                                        self->format_stats_instance(self));
 }
 
-static void
-_init_stats_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key)
+void
+log_threaded_dest_worker_written_bytes_add(LogThreadedDestWorker *self, gsize b)
 {
-  enum { labels_len = 2 };
-  static StatsClusterLabel labels[labels_len];
-
-  labels[0] = stats_cluster_label("id", self->super.super.id ? : "");
-  labels[1] = stats_cluster_label("driver_instance", self->format_stats_instance(self));
-  stats_cluster_logpipe_key_set(sc_key, "output_events_total", labels, labels_len);
-  stats_cluster_logpipe_key_add_legacy_alias(sc_key, self->stats_source | SCS_DESTINATION,
-                                             self->super.super.id,
-                                             self->format_stats_instance(self));
+  stats_byte_counter_add(&self->metrics.written_bytes, b);
 }
 
 void
 log_threaded_dest_driver_insert_msg_length_stats(LogThreadedDestDriver *self, gsize len)
 {
-  stats_aggregator_insert_data(self->max_message_size, len);
-  stats_aggregator_insert_data(self->average_messages_size, len);
+  stats_aggregator_insert_data(self->metrics.max_message_size, len);
+  stats_aggregator_insert_data(self->metrics.average_messages_size, len);
 }
 
 void
 log_threaded_dest_driver_insert_batch_length_stats(LogThreadedDestDriver *self, gsize len)
 {
-  stats_aggregator_insert_data(self->max_batch_size, len);
-  stats_aggregator_insert_data(self->average_batch_size, len);
+  stats_aggregator_insert_data(self->metrics.max_batch_size, len);
+  stats_aggregator_insert_data(self->metrics.average_batch_size, len);
 }
 
 void
 log_threaded_dest_driver_register_aggregated_stats(LogThreadedDestDriver *self)
 {
+  gint level = log_pipe_is_internal(&self->super.super.super) ? STATS_LEVEL3 : STATS_LEVEL0;
+
   StatsClusterKey sc_key_eps_input;
   _init_stats_legacy_key(self, &sc_key_eps_input);
   stats_aggregator_lock();
@@ -1012,23 +1033,23 @@ log_threaded_dest_driver_register_aggregated_stats(LogThreadedDestDriver *self)
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
                                                 self->format_stats_instance(self), "msg_size_max");
-  stats_register_aggregator_maximum(0, &sc_key, &self->max_message_size);
+  stats_register_aggregator_maximum(level, &sc_key, &self->metrics.max_message_size);
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
                                                 self->format_stats_instance(self), "msg_size_avg");
-  stats_register_aggregator_average(0, &sc_key, &self->average_messages_size);
+  stats_register_aggregator_average(level, &sc_key, &self->metrics.average_messages_size);
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
                                                 self->format_stats_instance(self), "batch_size_max");
-  stats_register_aggregator_maximum(0, &sc_key, &self->max_batch_size);
+  stats_register_aggregator_maximum(level, &sc_key, &self->metrics.max_batch_size);
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
                                                 self->format_stats_instance(self), "batch_size_avg");
-  stats_register_aggregator_average(0, &sc_key, &self->average_batch_size);
+  stats_register_aggregator_average(level, &sc_key, &self->metrics.average_batch_size);
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
                                                 self->format_stats_instance(self), "eps");
-  stats_register_aggregator_cps(0, &sc_key, &sc_key_eps_input, SC_TYPE_WRITTEN, &self->CPS);
+  stats_register_aggregator_cps(level, &sc_key, &sc_key_eps_input, SC_TYPE_WRITTEN, &self->metrics.CPS);
 
   stats_aggregator_unlock();
 }
@@ -1038,48 +1059,74 @@ log_threaded_dest_driver_unregister_aggregated_stats(LogThreadedDestDriver *self
 {
   stats_aggregator_lock();
 
-  stats_unregister_aggregator_maximum(&self->max_message_size);
-  stats_unregister_aggregator_average(&self->average_messages_size);
-  stats_unregister_aggregator_maximum(&self->max_batch_size);
-  stats_unregister_aggregator_average(&self->average_batch_size);
-  stats_unregister_aggregator_cps(&self->CPS);
+  stats_unregister_aggregator_maximum(&self->metrics.max_message_size);
+  stats_unregister_aggregator_average(&self->metrics.average_messages_size);
+  stats_unregister_aggregator_maximum(&self->metrics.max_batch_size);
+  stats_unregister_aggregator_average(&self->metrics.average_batch_size);
+  stats_unregister_aggregator_cps(&self->metrics.CPS);
 
   stats_aggregator_unlock();
 }
 
 static void
-_register_stats(LogThreadedDestDriver *self)
+_register_driver_stats(LogThreadedDestDriver *self, StatsClusterKeyBuilder *driver_sck_builder)
 {
+  if (!driver_sck_builder)
+    return;
+
+  gint level = log_pipe_is_internal(&self->super.super.super) ? STATS_LEVEL3 : STATS_LEVEL0;
+
+  stats_cluster_key_builder_set_name(driver_sck_builder, "output_events_total");
+  self->metrics.output_events_sc_key = stats_cluster_key_builder_build_logpipe(driver_sck_builder);
+
+  stats_cluster_key_builder_reset(driver_sck_builder);
+  stats_cluster_key_builder_set_legacy_alias(driver_sck_builder, self->stats_source | SCS_DESTINATION,
+                                             self->super.super.id, self->format_stats_instance(self));
+  stats_cluster_key_builder_set_legacy_alias_name(driver_sck_builder, "processed");
+  self->metrics.processed_sc_key = stats_cluster_key_builder_build_single(driver_sck_builder);
+
   stats_lock();
   {
-    StatsClusterKey sc_key;
-
-    _init_stats_key(self, &sc_key);
-    stats_register_counter(0, &sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-    stats_register_counter(0, &sc_key, SC_TYPE_WRITTEN, &self->written_messages);
-
-    stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
-                                                  self->format_stats_instance(self), "processed");
-    stats_register_counter(0, &sc_key, SC_TYPE_SINGLE_VALUE, &self->processed_messages);
-
+    stats_register_counter(level, self->metrics.output_events_sc_key, SC_TYPE_DROPPED, &self->metrics.dropped_messages);
+    stats_register_counter(level, self->metrics.output_events_sc_key, SC_TYPE_WRITTEN, &self->metrics.written_messages);
+    stats_register_counter(level, self->metrics.processed_sc_key, SC_TYPE_SINGLE_VALUE,
+                           &self->metrics.processed_messages);
   }
   stats_unlock();
 }
 
 static void
-_unregister_stats(LogThreadedDestDriver *self)
+_init_driver_sck_builder(LogThreadedDestDriver *self, StatsClusterKeyBuilder *builder)
+{
+  stats_cluster_key_builder_add_label(builder, stats_cluster_label("id", self->super.super.id ? : ""));
+  stats_cluster_key_builder_add_label(builder,
+                                      stats_cluster_label("driver_instance", self->format_stats_instance(self)));
+  stats_cluster_key_builder_set_legacy_alias(builder, self->stats_source | SCS_DESTINATION,
+                                             self->super.super.id,
+                                             self->format_stats_instance(self));
+}
+
+static void
+_unregister_driver_stats(LogThreadedDestDriver *self)
 {
   stats_lock();
   {
-    StatsClusterKey sc_key;
+    if (self->metrics.output_events_sc_key)
+      {
+        stats_unregister_counter(self->metrics.output_events_sc_key, SC_TYPE_DROPPED, &self->metrics.dropped_messages);
+        stats_unregister_counter(self->metrics.output_events_sc_key, SC_TYPE_WRITTEN, &self->metrics.written_messages);
 
-    _init_stats_key(self, &sc_key);
-    stats_unregister_counter(&sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-    stats_unregister_counter(&sc_key, SC_TYPE_WRITTEN, &self->written_messages);
+        stats_cluster_key_free(self->metrics.output_events_sc_key);
+        self->metrics.output_events_sc_key = NULL;
+      }
 
-    stats_cluster_single_key_legacy_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
-                                                  self->format_stats_instance(self), "processed");
-    stats_unregister_counter(&sc_key, SC_TYPE_SINGLE_VALUE, &self->processed_messages);
+    if (self->metrics.processed_sc_key)
+      {
+        stats_unregister_counter(self->metrics.processed_sc_key, SC_TYPE_SINGLE_VALUE, &self->metrics.processed_messages);
+
+        stats_cluster_key_free(self->metrics.processed_sc_key);
+        self->metrics.processed_sc_key = NULL;
+      }
   }
   stats_unlock();
 }
@@ -1096,7 +1143,7 @@ _format_seqnum_persist_name(LogThreadedDestDriver *self)
 }
 
 static gboolean
-_create_workers(LogThreadedDestDriver *self)
+_create_workers(LogThreadedDestDriver *self, gint stats_level, const StatsClusterKeyBuilder *driver_sck_builder)
 {
   /* free previous workers array if set to cope with num_workers change */
   g_free(self->workers);
@@ -1107,7 +1154,7 @@ _create_workers(LogThreadedDestDriver *self)
       LogThreadedDestWorker *dw = _construct_worker(self, self->created_workers);
 
       self->workers[self->created_workers] = dw;
-      if (!_acquire_worker_queue(dw))
+      if (!_acquire_worker_queue(dw, stats_level, driver_sck_builder))
         return FALSE;
     }
 
@@ -1143,11 +1190,19 @@ log_threaded_dest_driver_init_method(LogPipe *s)
   if (!self->shared_seq_num)
     init_sequence_number(&self->shared_seq_num);
 
-  _register_stats(self);
+  StatsClusterKeyBuilder *driver_sck_builder = stats_cluster_key_builder_new();
+  _init_driver_sck_builder(self, driver_sck_builder);
 
-  if (!_create_workers(self))
-    return FALSE;
+  gint stats_level = log_pipe_is_internal(&self->super.super.super) ? STATS_LEVEL3 : STATS_LEVEL0;
+  if (!_create_workers(self, stats_level, driver_sck_builder))
+    {
+      stats_cluster_key_builder_free(driver_sck_builder);
+      return FALSE;
+    }
 
+  _register_driver_stats(self, driver_sck_builder);
+
+  stats_cluster_key_builder_free(driver_sck_builder);
   return TRUE;
 }
 
@@ -1178,9 +1233,9 @@ log_threaded_dest_driver_deinit_method(LogPipe *s)
 
   cfg_persist_config_add(log_pipe_get_config(s),
                          _format_seqnum_persist_name(self),
-                         GINT_TO_POINTER(self->shared_seq_num), NULL, FALSE);
+                         GINT_TO_POINTER(self->shared_seq_num), NULL);
 
-  _unregister_stats(self);
+  _unregister_driver_stats(self);
 
   if (!_is_worker_compat_mode(self))
     {
