@@ -23,58 +23,39 @@
  */
 
 #include "logthrsourcedrv.h"
-#include "mainloop-worker.h"
+#include "mainloop-threaded-worker.h"
 #include "messages.h"
 #include "apphook.h"
 #include "ack-tracker/ack_tracker_factory.h"
 
 #include <iv.h>
 
-typedef struct _WakeupCondition
-{
-  GMutex *lock;
-  GCond *cond;
-  gboolean awoken;
-} WakeupCondition;
-
-struct _LogThreadedSourceWorker
-{
-  LogSource super;
-  LogThreadedSourceDriver *control;
-  WakeupCondition wakeup_cond;
-  WorkerOptions options;
-  gboolean under_termination;
-
-  LogThreadedSourceWorkerRunFunc run;
-  LogThreadedSourceWorkerRequestExitFunc request_exit;
-  LogThreadedSourceWorkerWakeupFunc wakeup;
-};
 
 static void
 wakeup_cond_init(WakeupCondition *cond)
 {
-  cond->lock = g_mutex_new();
-  cond->cond = g_cond_new();
+  g_mutex_init(&cond->lock);
+  g_cond_init(&cond->cond);
   cond->awoken = TRUE;
 }
 
 static void
 wakeup_cond_destroy(WakeupCondition *cond)
 {
-  g_cond_free(cond->cond);
-  g_mutex_free(cond->lock);
+  g_cond_clear(&cond->cond);
+  g_mutex_clear(&cond->lock);
 }
 
 static inline void
 wakeup_cond_lock(WakeupCondition *cond)
 {
-  g_mutex_lock(cond->lock);
+  g_mutex_lock(&cond->lock);
 }
 
 static inline void
 wakeup_cond_unlock(WakeupCondition *cond)
 {
-  g_mutex_unlock(cond->lock);
+  g_mutex_unlock(&cond->lock);
 }
 
 /* The wakeup lock must be held before calling this function. */
@@ -83,16 +64,16 @@ wakeup_cond_wait(WakeupCondition *cond)
 {
   cond->awoken = FALSE;
   while (!cond->awoken)
-    g_cond_wait(cond->cond, cond->lock);
+    g_cond_wait(&cond->cond, &cond->lock);
 }
 
 static inline void
 wakeup_cond_signal(WakeupCondition *cond)
 {
-  g_mutex_lock(cond->lock);
+  g_mutex_lock(&cond->lock);
   cond->awoken = TRUE;
-  g_cond_signal(cond->cond);
-  g_mutex_unlock(cond->lock);
+  g_cond_signal(&cond->cond);
+  g_mutex_unlock(&cond->lock);
 }
 
 static LogPipe *
@@ -158,27 +139,48 @@ log_threaded_source_wakeup(LogThreadedSourceDriver *self)
   wakeup_cond_signal(&worker->wakeup_cond);
 }
 
-static void
-log_threaded_source_worker_run(LogThreadedSourceWorker *self)
+static gboolean
+log_threaded_source_worker_thread_init(MainLoopThreadedWorker *s)
 {
-  msg_debug("Worker thread started", evt_tag_str("driver", self->control->super.super.id));
+  LogThreadedSourceWorker *self = (LogThreadedSourceWorker *) s->data;
 
-  /* ivykis is not used here, but mark-freq() requires all source threads to be iv-initialized. */
-  iv_init();
-
-  self->run(self->control);
-
-  iv_deinit();
-
-  msg_debug("Worker thread finished", evt_tag_str("driver", self->control->super.super.id));
+  if (self->control->thread_init)
+    return self->control->thread_init(self->control);
+  return TRUE;
 }
 
 static void
-log_threaded_source_worker_request_exit(LogThreadedSourceWorker *self)
+log_threaded_source_worker_thread_deinit(MainLoopThreadedWorker *s)
 {
-  msg_debug("Requesting worker thread exit", evt_tag_str("driver", self->control->super.super.id));
+  LogThreadedSourceWorker *self = (LogThreadedSourceWorker *) s->data;
+
+  if (self->control->thread_deinit)
+    self->control->thread_deinit(self->control);
+}
+
+static void
+log_threaded_source_worker_run(MainLoopThreadedWorker *s)
+{
+  LogThreadedSourceWorker *self = (LogThreadedSourceWorker *) s->data;
+
+  msg_debug("Worker thread started",
+            evt_tag_str("driver", self->control->super.super.id));
+
+  self->control->run(self->control);
+
+  msg_debug("Worker thread finished",
+            evt_tag_str("driver", self->control->super.super.id));
+}
+
+static void
+log_threaded_source_worker_request_exit(MainLoopThreadedWorker *s)
+{
+  LogThreadedSourceWorker *self = (LogThreadedSourceWorker *) s->data;
+
+  msg_debug("Requesting worker thread exit",
+            evt_tag_str("driver", self->control->super.super.id));
   self->under_termination = TRUE;
-  self->request_exit(self->control);
+  self->control->request_exit(self->control);
   log_threaded_source_wakeup(self->control);
 }
 
@@ -187,7 +189,7 @@ _worker_wakeup(LogSource *s)
 {
   LogThreadedSourceWorker *self = (LogThreadedSourceWorker *) s;
 
-  self->wakeup(self->control);
+  self->control->wakeup(self->control);
 }
 
 static gboolean
@@ -209,6 +211,7 @@ log_threaded_source_worker_free(LogPipe *s)
   log_pipe_unref(&self->control->super.super.super);
   self->control = NULL;
 
+  main_loop_threaded_worker_clear(&self->thread);
   log_source_free(s);
 }
 
@@ -217,11 +220,14 @@ log_threaded_source_worker_new(GlobalConfig *cfg)
 {
   LogThreadedSourceWorker *self = g_new0(LogThreadedSourceWorker, 1);
   log_source_init_instance(&self->super, cfg);
+  main_loop_threaded_worker_init(&self->thread, MLW_THREADED_INPUT_WORKER, self);
+  self->thread.thread_init = log_threaded_source_worker_thread_init;
+  self->thread.thread_deinit = log_threaded_source_worker_thread_deinit;
+  self->thread.run = log_threaded_source_worker_run;
+  self->thread.request_exit = log_threaded_source_worker_request_exit;
 
   wakeup_cond_init(&self->wakeup_cond);
 
-  self->options.is_external_input = TRUE;
-
   self->super.super.init = log_threaded_source_worker_init;
   self->super.super.free_fn = log_threaded_source_worker_free;
   self->super.wakeup = _worker_wakeup;
@@ -229,6 +235,13 @@ log_threaded_source_worker_new(GlobalConfig *cfg)
   return self;
 }
 
+gboolean
+log_threaded_source_driver_pre_config_init(LogPipe *s)
+{
+  main_loop_worker_allocate_thread_space(1);
+  return TRUE;
+}
+
 gboolean
 log_threaded_source_driver_init_method(LogPipe *s)
 {
@@ -236,7 +249,6 @@ log_threaded_source_driver_init_method(LogPipe *s)
   GlobalConfig *cfg = log_pipe_get_config(s);
 
   self->worker = log_threaded_source_worker_new(cfg);
-  self->worker->wakeup = log_threaded_source_wakeup;
 
   if (!log_src_driver_init_method(s))
     return FALSE;
@@ -286,32 +298,11 @@ log_threaded_source_driver_start_worker(LogPipe *s)
 {
   LogThreadedSourceDriver *self = (LogThreadedSourceDriver *) s;
 
-  main_loop_create_worker_thread((WorkerThreadFunc) log_threaded_source_worker_run,
-                                 (WorkerExitNotificationFunc) log_threaded_source_worker_request_exit,
-                                 self->worker, &self->worker->options);
+  main_loop_threaded_worker_start(&self->worker->thread);
 
   return TRUE;
 }
 
-void
-log_threaded_source_driver_set_worker_run_func(LogThreadedSourceDriver *self, LogThreadedSourceWorkerRunFunc run)
-{
-  self->worker->run = run;
-}
-
-void
-log_threaded_source_driver_set_worker_request_exit_func(LogThreadedSourceDriver *self,
-                                                        LogThreadedSourceWorkerRequestExitFunc request_exit)
-{
-  self->worker->request_exit = request_exit;
-}
-
-void
-log_threaded_source_set_wakeup_func(LogThreadedSourceDriver *self, LogThreadedSourceWorkerWakeupFunc wakeup)
-{
-  self->worker->wakeup = wakeup;
-}
-
 static gboolean
 _is_default_priority_or_facility_set(MsgFormatOptions *parse_options)
 {
@@ -327,12 +318,37 @@ _apply_default_priority_and_facility(LogThreadedSourceDriver *self, LogMessage *
   msg->pri = parse_options->default_pri;
 }
 
+/*
+ * Call this every some messages so consumers that accumulate multiple
+ * messages (LogQueueFifo for instance) can finish accumulation and go on
+ * processing a batch.
+ *
+ * Basically this calls main_loop_worker_invoke_batch_callbacks(), which is
+ * done by the minaloop-io-worker layer whenever we go back to the main
+ * loop.  Whether this is done automatically by LogThreadedSourceDriver is
+ * controlled by the auto_close_batches member, in which case we do this
+ * every message.
+ *
+ * Doing it every message defeats the purpose more or less, as consumers
+ * tend to do batching to improve performance.
+ */
+void
+log_threaded_source_close_batch(LogThreadedSourceDriver *self)
+{
+  main_loop_worker_invoke_batch_callbacks();
+}
+
 void
 log_threaded_source_post(LogThreadedSourceDriver *self, LogMessage *msg)
 {
-  msg_debug("Incoming log message", evt_tag_str("msg", log_msg_get_value(msg, LM_V_MESSAGE, NULL)));
+  msg_debug("Incoming log message",
+            evt_tag_str("input", log_msg_get_value(msg, LM_V_MESSAGE, NULL)),
+            evt_tag_msg_reference(msg));
   _apply_default_priority_and_facility(self, msg);
   log_source_post(&self->worker->super, msg);
+
+  if (self->auto_close_batches)
+    log_threaded_source_close_batch(self);
 }
 
 gboolean
@@ -374,5 +390,10 @@ log_threaded_source_driver_init_instance(LogThreadedSourceDriver *self, GlobalCo
   self->super.super.super.init = log_threaded_source_driver_init_method;
   self->super.super.super.deinit = log_threaded_source_driver_deinit_method;
   self->super.super.super.free_fn = log_threaded_source_driver_free_method;
-  self->super.super.super.on_config_inited = log_threaded_source_driver_start_worker;
+  self->super.super.super.pre_config_init = log_threaded_source_driver_pre_config_init;
+  self->super.super.super.post_config_init = log_threaded_source_driver_start_worker;
+
+  self->wakeup = log_threaded_source_wakeup;
+
+  self->auto_close_batches = TRUE;
 }
