@@ -28,6 +28,7 @@
 #include "stats/aggregator/stats-aggregator-registry.h"
 #include "stats/stats-cluster-single.h"
 #include "stats/aggregator/stats-aggregator.h"
+#include "stats/stats-compat.h"
 #include "hostname.h"
 #include "host-resolve.h"
 #include "seqnum.h"
@@ -68,18 +69,27 @@ struct _LogWriter
   guint32 flags:31;
   gint32 seq_num;
   gboolean partial_write;
-  StatsCounterItem *dropped_messages;
-  StatsCounterItem *suppressed_messages;
-  StatsCounterItem *processed_messages;
-  StatsCounterItem *written_messages;
-  StatsAggregator *max_message_size;
-  StatsAggregator *average_messages_size;
-  StatsAggregator *CPS;
+
   struct
   {
-    StatsCounterItem *count;
-    StatsCounterItem *bytes;
-  } truncated;
+    StatsCounterItem *dropped_messages;
+    StatsCounterItem *suppressed_messages;
+    StatsCounterItem *processed_messages;
+    StatsCounterItem *written_messages;
+
+    StatsByteCounter written_bytes;
+
+    StatsAggregator *max_message_size;
+    StatsAggregator *average_messages_size;
+    StatsAggregator *CPS;
+
+    struct
+    {
+      StatsCounterItem *count;
+      StatsCounterItem *bytes;
+    } truncated;
+  } metrics;
+
   LogPipe *control;
   LogWriterOptions *options;
   LogMessage *last_msg;
@@ -196,9 +206,10 @@ log_writer_set_queue(LogWriter *self, LogQueue *queue)
 }
 
 static void
-log_writer_work_perform(gpointer s, GIOCondition cond)
+log_writer_work_perform(gpointer s, gpointer arg)
 {
   LogWriter *self = (LogWriter *) s;
+  GIOCondition cond = GPOINTER_TO_INT(arg);
 
   g_assert((self->super.flags & PIF_INITIALIZED) != 0);
   g_assert((cond == G_IO_OUT) || (cond == G_IO_IN));
@@ -210,7 +221,7 @@ log_writer_work_perform(gpointer s, GIOCondition cond)
 }
 
 static void
-log_writer_work_finished(gpointer s)
+log_writer_work_finished(gpointer s, gpointer arg)
 {
   LogWriter *self = (LogWriter *) s;
 
@@ -264,7 +275,7 @@ log_writer_io_handler(gpointer s, GIOCondition cond)
   log_writer_stop_watches(self);
   if ((self->options->options & LWO_THREADED))
     {
-      main_loop_io_worker_job_submit(&self->io_job, cond);
+      main_loop_io_worker_job_submit(&self->io_job, GINT_TO_POINTER(cond));
     }
   else
     {
@@ -280,8 +291,8 @@ log_writer_io_handler(gpointer s, GIOCondition cond)
       if (!main_loop_worker_job_quit())
         {
           log_pipe_ref(&self->super);
-          log_writer_work_perform(s, cond);
-          log_writer_work_finished(s);
+          log_writer_work_perform(s, GINT_TO_POINTER(cond));
+          log_writer_work_finished(s, NULL);
           log_pipe_unref(&self->super);
         }
     }
@@ -749,7 +760,7 @@ log_writer_is_msg_suppressed(LogWriter *self, LogMessage *lm)
           !_is_message_a_mark(lm))
         {
 
-          stats_counter_inc(self->suppressed_messages);
+          stats_counter_inc(self->metrics.suppressed_messages);
           self->last_msg_count++;
 
           /* we only create the timer if this is the first suppressed message, otherwise it is already running. */
@@ -813,7 +824,7 @@ log_writer_mark_timeout(void *cookie)
   if (!log_writer_is_msg_suppressed(self, msg))
     {
       log_queue_push_tail(self->queue, msg, &path_options);
-      stats_counter_inc(self->processed_messages);
+      stats_counter_inc(self->metrics.processed_messages);
     }
   else
     {
@@ -865,7 +876,7 @@ log_writer_queue(LogPipe *s, LogMessage *lm, const LogPathOptions *path_options)
       log_writer_postpone_mark_timer(self);
     }
 
-  stats_counter_inc(self->processed_messages);
+  stats_counter_inc(self->metrics.processed_messages);
   log_queue_push_tail(self->queue, lm, path_options);
 }
 
@@ -1113,8 +1124,8 @@ log_writer_format_log(LogWriter *self, LogMessage *lm, GString *result)
 
       g_string_truncate(result, self->options->truncate_size);
 
-      stats_counter_inc(self->truncated.count);
-      stats_counter_add(self->truncated.bytes, truncated_bytes);
+      stats_counter_inc(self->metrics.truncated.count);
+      stats_counter_add(self->metrics.truncated.bytes, truncated_bytes);
     }
 }
 
@@ -1160,8 +1171,8 @@ log_writer_flush_finalize(LogWriter *self)
 static void
 _log_writer_insert_msg_length_stats(LogWriter *self, gsize msg_len)
 {
-  stats_aggregator_insert_data(self->max_message_size, msg_len);
-  stats_aggregator_insert_data(self->average_messages_size, msg_len);
+  stats_aggregator_insert_data(self->metrics.max_message_size, msg_len);
+  stats_aggregator_insert_data(self->metrics.average_messages_size, msg_len);
 }
 
 static gboolean
@@ -1227,6 +1238,7 @@ log_writer_write_message(LogWriter *self, LogMessage *msg, LogPathOptions *path_
       log_msg_unref(msg);
       msg_set_context(NULL);
       log_msg_refcache_stop();
+      stats_byte_counter_add(&self->metrics.written_bytes, msg_len);
       _log_writer_insert_msg_length_stats(self, msg_len);
 
       return TRUE;
@@ -1311,7 +1323,7 @@ log_writer_flush(LogWriter *self, LogWriterFlushMode flush_mode)
       scratch_buffers_reclaim_marked(mark);
 
       if (!write_error)
-        stats_counter_inc(self->written_messages);
+        stats_counter_inc(self->metrics.written_messages);
     }
 
   if (write_error)
@@ -1400,29 +1412,29 @@ log_writer_init_watches(LogWriter *self)
 
   main_loop_io_worker_job_init(&self->io_job);
   self->io_job.user_data = self;
-  self->io_job.work = (void (*)(void *, GIOCondition)) log_writer_work_perform;
-  self->io_job.completion = (void (*)(void *)) log_writer_work_finished;
+  self->io_job.work = (void (*)(void *, void *)) log_writer_work_perform;
+  self->io_job.completion = (void (*)(void *, void *)) log_writer_work_finished;
   self->io_job.engage = (void (*)(void *)) log_pipe_ref;
   self->io_job.release = (void (*)(void *)) log_pipe_unref;
 }
 
 static void
-_register_aggregated_stats(LogWriter *self, StatsClusterKey *sc_key_input, gint stats_type)
+_register_aggregated_stats(LogWriter *self, StatsClusterKey *sc_key_input, gint stats_level, gint stats_type)
 {
   stats_aggregator_lock();
   StatsClusterKey sc_key;
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->options->stats_source | SCS_DESTINATION, self->stats_id,
                                                 self->stats_instance, "msg_size_max");
-  stats_register_aggregator_maximum(self->options->stats_level, &sc_key, &self->max_message_size);
+  stats_register_aggregator_maximum(stats_level, &sc_key, &self->metrics.max_message_size);
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->options->stats_source | SCS_DESTINATION, self->stats_id,
                                                 self->stats_instance, "msg_size_avg");
-  stats_register_aggregator_average(self->options->stats_level, &sc_key, &self->average_messages_size);
+  stats_register_aggregator_average(stats_level, &sc_key, &self->metrics.average_messages_size);
 
   stats_cluster_single_key_legacy_set_with_name(&sc_key, self->options->stats_source | SCS_DESTINATION, self->stats_id,
                                                 self->stats_instance, "eps");
-  stats_register_aggregator_cps(self->options->stats_level, &sc_key, sc_key_input, stats_type, &self->CPS);
+  stats_register_aggregator_cps(stats_level, &sc_key, sc_key_input, stats_type, &self->metrics.CPS);
 
   stats_aggregator_unlock();
 }
@@ -1432,17 +1444,48 @@ _unregister_aggregated_stats(LogWriter *self)
 {
   stats_aggregator_lock();
 
-  stats_unregister_aggregator_maximum(&self->max_message_size);
-  stats_unregister_aggregator_average(&self->average_messages_size);
-  stats_unregister_aggregator_cps(&self->CPS);
+  stats_unregister_aggregator_maximum(&self->metrics.max_message_size);
+  stats_unregister_aggregator_average(&self->metrics.average_messages_size);
+  stats_unregister_aggregator_cps(&self->metrics.CPS);
 
   stats_aggregator_unlock();
 }
 
+static void
+_register_raw_bytes_stats(LogWriter *self, gint stats_level)
+{
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->stats_id),
+    stats_cluster_label("driver_instance", self->stats_instance),
+  };
+
+  StatsClusterKey output_bytes_key;
+  stats_cluster_single_key_set(&output_bytes_key, "output_event_bytes_total", labels, G_N_ELEMENTS(labels));
+  stats_byte_counter_init(&self->metrics.written_bytes, &output_bytes_key, stats_level, SBCP_KIB);
+}
+
+static void
+_unregister_raw_bytes_stats(LogWriter *self)
+{
+  StatsClusterLabel labels[] =
+  {
+    stats_cluster_label("id", self->stats_id),
+    stats_cluster_label("driver_instance", self->stats_instance),
+  };
+
+  StatsClusterKey output_bytes_key;
+  stats_cluster_single_key_set(&output_bytes_key, "output_event_bytes_total", labels, G_N_ELEMENTS(labels));
+  stats_byte_counter_deinit(&self->metrics.written_bytes, &output_bytes_key);
+}
+
 static void
 _register_counters(LogWriter *self)
 {
   stats_lock();
+
+  gint level = log_pipe_is_internal(&self->super) ? STATS_LEVEL3 : self->options->stats_level;
+
   StatsClusterKey sc_key;
   StatsClusterLabel labels[] =
   {
@@ -1454,31 +1497,30 @@ _register_counters(LogWriter *self)
                                              self->stats_instance);
 
   if (self->options->suppress > 0)
-    stats_register_counter(self->options->stats_level, &sc_key, SC_TYPE_SUPPRESSED, &self->suppressed_messages);
-  stats_register_counter(self->options->stats_level, &sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-  stats_register_counter(self->options->stats_level, &sc_key, SC_TYPE_WRITTEN, &self->written_messages);
-  log_queue_register_stats_counters(self->queue, self->options->stats_level, &sc_key);
+    stats_register_counter(level, &sc_key, SC_TYPE_SUPPRESSED, &self->metrics.suppressed_messages);
+  stats_register_counter(level, &sc_key, SC_TYPE_DROPPED, &self->metrics.dropped_messages);
+  stats_register_counter(level, &sc_key, SC_TYPE_WRITTEN, &self->metrics.written_messages);
 
   StatsClusterKey sc_legacy_processed;
   stats_cluster_single_key_legacy_set_with_name(&sc_legacy_processed, self->options->stats_source | SCS_DESTINATION,
                                                 self->stats_id, self->stats_instance, "processed");
-  stats_register_counter(self->options->stats_level, &sc_legacy_processed, SC_TYPE_SINGLE_VALUE,
-                         &self->processed_messages);
+  stats_register_counter(level, &sc_legacy_processed, SC_TYPE_SINGLE_VALUE, &self->metrics.processed_messages);
 
   StatsClusterKey sc_key_truncated_count;
   stats_cluster_single_key_legacy_set_with_name(&sc_key_truncated_count, self->options->stats_source | SCS_DESTINATION,
                                                 self->stats_id, self->stats_instance, "truncated_count");
-  stats_register_counter(self->options->stats_level, &sc_key_truncated_count, SC_TYPE_SINGLE_VALUE,
-                         &self->truncated.count);
+  stats_register_counter(level, &sc_key_truncated_count, SC_TYPE_SINGLE_VALUE, &self->metrics.truncated.count);
 
   StatsClusterKey sc_key_truncated_bytes;
   stats_cluster_single_key_legacy_set_with_name(&sc_key_truncated_bytes, self->options->stats_source | SCS_DESTINATION,
                                                 self->stats_id, self->stats_instance, "truncated_bytes");
-  stats_register_counter(self->options->stats_level, &sc_key_truncated_bytes, SC_TYPE_SINGLE_VALUE,
-                         &self->truncated.bytes);
+  stats_register_counter(level, &sc_key_truncated_bytes, SC_TYPE_SINGLE_VALUE, &self->metrics.truncated.bytes);
 
   stats_unlock();
-  _register_aggregated_stats(self, &sc_key, SC_TYPE_WRITTEN);
+  _register_aggregated_stats(self, &sc_key, level, SC_TYPE_WRITTEN);
+
+  level = log_pipe_is_internal(&self->super) ? STATS_LEVEL3 : STATS_LEVEL1;
+  _register_raw_bytes_stats(self, level);
 }
 
 static gboolean
@@ -1492,7 +1534,7 @@ log_writer_init(LogPipe *s)
     }
   iv_event_register(&self->queue_filled);
 
-  if ((self->options->options & LWO_NO_STATS) == 0 && !self->dropped_messages)
+  if ((self->options->options & LWO_NO_STATS) == 0 && !self->metrics.dropped_messages)
     _register_counters(self);
 
 
@@ -1522,31 +1564,28 @@ _unregister_counters(LogWriter *self)
     stats_cluster_logpipe_key_add_legacy_alias(&sc_key, self->options->stats_source | SCS_DESTINATION, self->stats_id,
                                                self->stats_instance);
 
-    stats_unregister_counter(&sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
-    stats_unregister_counter(&sc_key, SC_TYPE_SUPPRESSED, &self->suppressed_messages);
-    stats_unregister_counter(&sc_key, SC_TYPE_WRITTEN, &self->written_messages);
+    stats_unregister_counter(&sc_key, SC_TYPE_DROPPED, &self->metrics.dropped_messages);
+    stats_unregister_counter(&sc_key, SC_TYPE_SUPPRESSED, &self->metrics.suppressed_messages);
+    stats_unregister_counter(&sc_key, SC_TYPE_WRITTEN, &self->metrics.written_messages);
 
     StatsClusterKey sc_legacy_processed;
     stats_cluster_single_key_legacy_set_with_name(&sc_legacy_processed, self->options->stats_source | SCS_DESTINATION,
                                                   self->stats_id, self->stats_instance, "processed");
-    stats_unregister_counter(&sc_legacy_processed, SC_TYPE_SINGLE_VALUE, &self->processed_messages);
+    stats_unregister_counter(&sc_legacy_processed, SC_TYPE_SINGLE_VALUE, &self->metrics.processed_messages);
 
     StatsClusterKey sc_key_truncated_count;
     stats_cluster_single_key_legacy_set_with_name(&sc_key_truncated_count, self->options->stats_source | SCS_DESTINATION,
                                                   self->stats_id, self->stats_instance, "truncated_count");
-    stats_unregister_counter(&sc_key_truncated_count, SC_TYPE_SINGLE_VALUE, &self->truncated.count);
+    stats_unregister_counter(&sc_key_truncated_count, SC_TYPE_SINGLE_VALUE, &self->metrics.truncated.count);
 
     StatsClusterKey sc_key_truncated_bytes;
     stats_cluster_single_key_legacy_set_with_name(&sc_key_truncated_bytes, self->options->stats_source | SCS_DESTINATION,
                                                   self->stats_id, self->stats_instance, "truncated_bytes");
-    stats_unregister_counter(&sc_key_truncated_bytes, SC_TYPE_SINGLE_VALUE, &self->truncated.bytes);
-
-    log_queue_unregister_stats_counters(self->queue, &sc_key);
-
+    stats_unregister_counter(&sc_key_truncated_bytes, SC_TYPE_SINGLE_VALUE, &self->metrics.truncated.bytes);
   }
   stats_unlock();
   _unregister_aggregated_stats(self);
-
+  _unregister_raw_bytes_stats(self);
 }
 
 static gboolean
@@ -1751,6 +1790,22 @@ log_writer_reopen(LogWriter *s, LogProtoClient *proto)
     }
 }
 
+void
+log_writer_init_driver_sck_builder(LogWriter *self, StatsClusterKeyBuilder *builder)
+{
+  stats_cluster_key_builder_add_label(builder, stats_cluster_label("id", self->stats_id));
+  stats_cluster_key_builder_add_label(builder, stats_cluster_label("driver_instance", self->stats_instance));
+  stats_cluster_key_builder_set_legacy_alias(builder, self->options->stats_source | SCS_DESTINATION, self->stats_id,
+                                             self->stats_instance);
+}
+
+void
+log_writer_init_queue_sck_builder(LogWriter *self, StatsClusterKeyBuilder *builder)
+{
+  stats_cluster_key_builder_add_label(builder, stats_cluster_label("id", self->stats_id));
+  stats_cluster_key_builder_add_label(builder, stats_cluster_label("driver_instance", self->stats_instance));
+}
+
 void
 log_writer_set_options(LogWriter *self, LogPipe *control, LogWriterOptions *options,
                        const gchar *stats_id, const gchar *stats_instance)
