@@ -48,18 +48,19 @@ _engage(MainLoopIOWorkerJob *self)
 }
 
 /* NOTE: runs in the main thread */
-void
-main_loop_io_worker_job_submit(MainLoopIOWorkerJob *self, GIOCondition cond)
+gboolean
+main_loop_io_worker_job_submit(MainLoopIOWorkerJob *self, gpointer arg)
 {
   g_assert(self->working == FALSE);
   if (main_loop_workers_quit)
-    return;
+    return FALSE;
 
   _engage(self);
   main_loop_worker_job_start();
   self->working = TRUE;
-  self->cond = cond;
+  self->arg = arg;
   iv_work_pool_submit_work(&main_loop_io_workers, &self->work_item);
+  return TRUE;
 }
 
 /* NOTE: runs in the actual worker thread spawned by the
@@ -67,7 +68,7 @@ main_loop_io_worker_job_submit(MainLoopIOWorkerJob *self, GIOCondition cond)
 static void
 _work(MainLoopIOWorkerJob *self)
 {
-  self->work(self->user_data, self->cond);
+  self->work(self->user_data, self->arg);
   main_loop_worker_invoke_batch_callbacks();
   main_loop_worker_run_gc();
 }
@@ -77,7 +78,8 @@ static void
 _complete(MainLoopIOWorkerJob *self)
 {
   self->working = FALSE;
-  self->completion(self->user_data);
+  if (self->completion)
+    self->completion(self->user_data, self->arg);
   main_loop_worker_job_complete();
   _release(self);
 }
