@@ -147,42 +147,6 @@ vp_cmdline_parse_rekey(const gchar *option_name, const gchar *value,
   return TRUE;
 }
 
-static void
-value_pairs_parse_type(gchar *spec, gchar **value, gchar **type)
-{
-  char *sp, *ep;
-
-  *type = NULL;
-  sp = spec;
-
-  while (g_ascii_isalnum(*sp) || (*sp) == '_')
-    sp++;
-
-  while (*sp == ' ' || *sp == '\t')
-    sp++;
-
-  if (*sp != '(' ||
-      !((g_ascii_toupper(spec[0]) >= 'A' &&
-         g_ascii_toupper(spec[0]) <= 'Z') ||
-        spec[0] == '_'))
-    {
-      *value = spec;
-      return;
-    }
-
-  ep = strrchr(sp, ')');
-  if (ep == NULL || ep[1] != '\0')
-    {
-      *value = spec;
-      return;
-    }
-
-  *value = sp + 1;
-  *type = spec;
-  sp[0] = '\0';
-  ep[0] = '\0';
-}
-
 static gboolean
 vp_cmdline_parse_pair (const gchar *option_name, const gchar *value,
                        gpointer data, GError **error)
@@ -190,7 +154,7 @@ vp_cmdline_parse_pair (const gchar *option_name, const gchar *value,
   gpointer *args = (gpointer *) data;
   ValuePairs *vp = (ValuePairs *) args[1];
   GlobalConfig *cfg = (GlobalConfig *) args[0];
-  gchar **kv, *v, *t;
+  gchar **kv;
   gboolean res = FALSE;
   LogTemplate *template;
 
@@ -204,21 +168,9 @@ vp_cmdline_parse_pair (const gchar *option_name, const gchar *value,
     }
 
   kv = g_strsplit(value, "=", 2);
-  value_pairs_parse_type(kv[1], &v, &t);
 
   template = log_template_new(cfg, NULL);
-  if (!log_template_compile(template, v, error))
-    goto error;
-
-  /* NOTE: t == NULL means that we disable casting, which might be in place
-   * due to compatibility with 3.x.  This means that value-pairs would
-   * _expect_ log_template_format() to return a proper type even in 3.x
-   * mode.  This is OK as value pairs is doing its own compatibility
-   * guarantees and would override type information from LogTemplate.  This
-   * happens for instance with a --no-cast option to $(format-json), which
-   * explicitly asks for the new behavior.  */
-
-  if (!log_template_set_type_hint(template, t, error))
+  if (!log_template_compile_with_type_hint(template, kv[1], error))
     goto error;
 
   value_pairs_add_pair(vp, kv[0], template);
@@ -334,6 +286,48 @@ vp_cmdline_parse_rekey_add_prefix (const gchar *option_name, const gchar *value,
   return TRUE;
 }
 
+static gboolean
+vp_cmdline_parse_rekey_upper (const gchar *option_name, const gchar *value,
+                              gpointer data, GError **error)
+{
+  gpointer *args = (gpointer *) data;
+  ValuePairsTransformSet *vpts = (ValuePairsTransformSet *) args[2];
+  gchar *key = (gchar *) args[3];
+
+  vpts = vp_cmdline_rekey_verify (key, vpts, data);
+  if (!vpts)
+    {
+      g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_FAILED,
+                   "Error parsing value-pairs: --upper used without --key or --rekey");
+      return FALSE;
+    }
+
+  value_pairs_transform_set_add_func(vpts,
+                                     value_pairs_new_transform_upper ());
+  return TRUE;
+}
+
+static gboolean
+vp_cmdline_parse_rekey_lower (const gchar *option_name, const gchar *value,
+                              gpointer data, GError **error)
+{
+  gpointer *args = (gpointer *) data;
+  ValuePairsTransformSet *vpts = (ValuePairsTransformSet *) args[2];
+  gchar *key = (gchar *) args[3];
+
+  vpts = vp_cmdline_rekey_verify (key, vpts, data);
+  if (!vpts)
+    {
+      g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_FAILED,
+                   "Error parsing value-pairs: --lower used without --key or --rekey");
+      return FALSE;
+    }
+
+  value_pairs_transform_set_add_func(vpts,
+                                     value_pairs_new_transform_lower ());
+  return TRUE;
+}
+
 static gboolean
 vp_cmdline_parse_rekey_shift (const gchar *option_name, const gchar *value,
                               gpointer data, GError **error)
@@ -412,17 +406,13 @@ vp_cmdline_parse_cast(const gchar *option_name, const gchar *value,
   return TRUE;
 }
 
-ValuePairs *
-value_pairs_new_from_cmdline (GlobalConfig *cfg,
-                              gint *argc, gchar ***argv,
-                              gboolean ignore_unknown_options,
-                              GError **error)
+static gboolean
+value_pairs_parse_command_line(ValuePairs *vp,
+                               gint *argc, gchar ***argv,
+                               GOptionGroup *custom_options,
+                               GError **error)
 {
-  ValuePairs *vp;
   GOptionContext *ctx;
-
-  vp = value_pairs_new(cfg);
-
   GOptionEntry vp_options[] =
   {
     {
@@ -445,6 +435,14 @@ value_pairs_new_from_cmdline (GlobalConfig *cfg,
       "pair", 'p', 0, G_OPTION_ARG_CALLBACK, vp_cmdline_parse_pair,
       NULL, NULL
     },
+    {
+      "upper", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, vp_cmdline_parse_rekey_upper,
+      NULL, NULL
+    },
+    {
+      "lower", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, vp_cmdline_parse_rekey_lower,
+      NULL, NULL
+    },
     {
       "shift", 'S', 0, G_OPTION_ARG_CALLBACK, vp_cmdline_parse_rekey_shift,
       NULL, NULL
@@ -491,29 +489,45 @@ value_pairs_new_from_cmdline (GlobalConfig *cfg,
     },
     { NULL }
   };
-  GOptionGroup *og;
   gpointer user_data_args[4];
   gboolean success;
 
-  user_data_args[0] = cfg;
+  user_data_args[0] = vp->cfg;
   user_data_args[1] = vp;
   user_data_args[2] = NULL;
   user_data_args[3] = NULL;
 
   ctx = g_option_context_new("value-pairs");
-  og = g_option_group_new(NULL, NULL, NULL, user_data_args, NULL);
+  GOptionGroup *og = g_option_group_new("value-pairs", "", "", user_data_args, NULL);
   g_option_group_add_entries(og, vp_options);
+
+  /* only the main group gets to process G_OPTION_REMAINING options, so
+   * vp_options is the main one */
+
   g_option_context_set_main_group(ctx, og);
-  g_option_context_set_ignore_unknown_options(ctx, ignore_unknown_options);
+  if (custom_options)
+    g_option_context_add_group(ctx, custom_options);
 
   success = g_option_context_parse(ctx, argc, argv, error);
   vp_cmdline_parse_rekey_finish(user_data_args);
   g_option_context_free(ctx);
+  return success;
+}
 
-  if (!success)
+ValuePairs *
+value_pairs_new_from_cmdline(GlobalConfig *cfg,
+                             gint *argc, gchar ***argv,
+                             GOptionGroup *custom_options,
+                             GError **error)
+{
+  ValuePairs *vp;
+
+  vp = value_pairs_new(cfg);
+
+  if (!value_pairs_parse_command_line(vp, argc, argv, custom_options, error))
     {
       value_pairs_unref(vp);
-      vp = NULL;
+      return NULL;
     }
 
   return vp;
