@@ -91,7 +91,7 @@ static GList *affile_dest_drivers = NULL;
 struct _AFFileDestWriter
 {
   LogPipe super;
-  GStaticMutex lock;
+  GMutex lock;
   AFFileDestDriver *owner;
   gchar *filename;
   LogWriter *writer;
@@ -120,7 +120,7 @@ affile_dw_reap(AFFileDestWriter *self)
 
   main_loop_assert_main_thread();
 
-  g_static_mutex_lock(&owner->lock);
+  g_mutex_lock(&owner->lock);
   if (!log_writer_has_pending_writes((LogWriter *) self->writer) && !self->queue_pending)
     {
       msg_verbose("Destination timed out, reaping",
@@ -128,7 +128,7 @@ affile_dw_reap(AFFileDestWriter *self)
                   evt_tag_str("filename", self->filename));
       affile_dd_reap_writer(self->owner, self);
     }
-  g_static_mutex_unlock(&owner->lock);
+  g_mutex_unlock(&owner->lock);
 }
 
 static gboolean
@@ -140,7 +140,8 @@ affile_dw_reopen(AFFileDestWriter *self)
 
   msg_verbose("Initializing destination file writer",
               evt_tag_str("template", self->owner->filename_template->template),
-              evt_tag_str("filename", self->filename));
+              evt_tag_str("filename", self->filename),
+              evt_tag_str("symlink_as", self->owner->symlink_as));
 
   self->last_open_stamp = self->last_msg_stamp;
   if (self->owner->overwrite_if_older > 0 &&
@@ -156,6 +157,9 @@ affile_dw_reopen(AFFileDestWriter *self)
   FileOpenerResult open_result = file_opener_open_fd(self->owner->file_opener, self->filename, AFFILE_DIR_WRITE, &fd);
   if (open_result == FILE_OPENER_RESULT_SUCCESS)
     {
+      if (self->owner->symlink_as != NULL)
+        file_opener_symlink(self->owner->file_opener, self->owner->symlink_as, self->filename);
+
       LogTransport *transport = file_opener_construct_transport(self->owner->file_opener, fd);
 
       proto = file_opener_construct_dst_proto(self->owner->file_opener, transport,
@@ -188,13 +192,25 @@ affile_dw_init(LogPipe *s)
       self->writer = log_writer_new(self->owner->writer_flags, cfg);
     }
 
+  log_pipe_set_options((LogPipe *) self->writer, &s->options);
   log_writer_set_options(self->writer,
                          s,
                          &self->owner->writer_options,
                          self->owner->super.super.id,
                          self->filename);
-  log_writer_set_queue(self->writer, log_dest_driver_acquire_queue(&self->owner->super,
-                       affile_dw_format_persist_name(self)));
+
+  StatsClusterKeyBuilder *driver_sck_builder = stats_cluster_key_builder_new();
+  StatsClusterKeyBuilder *queue_sck_builder = stats_cluster_key_builder_new();
+  log_writer_init_driver_sck_builder(self->writer, driver_sck_builder);
+  log_writer_init_queue_sck_builder(self->writer, queue_sck_builder);
+
+  gint stats_level = log_pipe_is_internal(&self->super) ? STATS_LEVEL3 : self->owner->writer_options.stats_level;
+  LogQueue *queue = log_dest_driver_acquire_queue(&self->owner->super, affile_dw_format_persist_name(self),
+                                                  stats_level, driver_sck_builder, queue_sck_builder);
+  log_writer_set_queue(self->writer, queue);
+
+  stats_cluster_key_builder_free(driver_sck_builder);
+  stats_cluster_key_builder_free(queue_sck_builder);
 
   if (!log_pipe_init((LogPipe *) self->writer))
     {
@@ -251,7 +267,7 @@ affile_dw_queue(LogPipe *s, LogMessage *lm, const LogPathOptions *path_options)
 
   AFFileDestWriter *self = (AFFileDestWriter *) s;
 
-  g_static_mutex_lock(&self->lock);
+  g_mutex_lock(&self->lock);
   self->last_msg_stamp = cached_g_current_time_sec();
   if (self->last_open_stamp == 0)
     self->last_open_stamp = self->last_msg_stamp;
@@ -262,12 +278,12 @@ affile_dw_queue(LogPipe *s, LogMessage *lm, const LogPathOptions *path_options)
     {
       self->reopen_pending = TRUE;
       /* if the file couldn't be opened, try it again every time_reopen seconds */
-      g_static_mutex_unlock(&self->lock);
+      g_mutex_unlock(&self->lock);
       affile_dw_reopen(self);
-      g_static_mutex_lock(&self->lock);
+      g_mutex_lock(&self->lock);
       self->reopen_pending = FALSE;
     }
-  g_static_mutex_unlock(&self->lock);
+  g_mutex_unlock(&self->lock);
 
   log_pipe_forward_msg(&self->super, lm, path_options);
 }
@@ -283,6 +299,7 @@ affile_dw_set_owner(AFFileDestWriter *self, AFFileDestDriver *owner)
   self->owner = owner;
   self->super.expr_node = owner->super.super.super.expr_node;
 
+  log_pipe_set_options(&self->super, &owner->super.super.super.options);
   log_pipe_set_config(&self->super, cfg);
   if (self->writer)
     {
@@ -302,7 +319,7 @@ affile_dw_free(LogPipe *s)
 
   log_pipe_unref((LogPipe *) self->writer);
 
-  g_static_mutex_free(&self->lock);
+  g_mutex_clear(&self->lock);
   self->writer = NULL;
   g_free(self->filename);
   log_pipe_unref(&self->owner->super.super.super);
@@ -342,7 +359,7 @@ affile_dw_new(const gchar *filename, GlobalConfig *cfg)
   /* we have to take care about freeing filename later.
      This avoids a move of the filename. */
   self->filename = g_strdup(filename);
-  g_static_mutex_init(&self->lock);
+  g_mutex_init(&self->lock);
   return self;
 }
 
@@ -385,6 +402,15 @@ affile_dd_set_overwrite_if_older(LogDriver *s, gint overwrite_if_older)
   self->overwrite_if_older = overwrite_if_older;
 }
 
+void
+affile_dd_set_symlink_as(LogDriver *s, const gchar *symlink_as)
+{
+  AFFileDestDriver *self = (AFFileDestDriver *) s;
+
+  g_free(self->symlink_as);
+  self->symlink_as = g_strdup(symlink_as);
+}
+
 void
 affile_dd_set_fsync(LogDriver *s, gboolean use_fsync)
 {
@@ -572,7 +598,7 @@ affile_dd_deinit(LogPipe *s)
 
       log_pipe_deinit(&self->single_writer->super);
       cfg_persist_config_add(cfg, affile_dd_format_persist_name(s), self->single_writer,
-                             affile_dd_destroy_writer, FALSE);
+                             affile_dd_destroy_writer);
       self->single_writer = NULL;
     }
   else if (self->writer_hash)
@@ -581,7 +607,7 @@ affile_dd_deinit(LogPipe *s)
 
       g_hash_table_foreach(self->writer_hash, affile_dd_deinit_writer, NULL);
       cfg_persist_config_add(cfg, affile_dd_format_persist_name(s), self->writer_hash,
-                             affile_dd_destroy_writer_hash, FALSE);
+                             affile_dd_destroy_writer_hash);
       self->writer_hash = NULL;
     }
 
@@ -613,9 +639,9 @@ affile_dd_open_writer(gpointer args[])
           if (next && log_pipe_init(&next->super))
             {
               log_pipe_ref(&next->super);
-              g_static_mutex_lock(&self->lock);
+              g_mutex_lock(&self->lock);
               self->single_writer = next;
-              g_static_mutex_unlock(&self->lock);
+              g_mutex_unlock(&self->lock);
             }
           else
             {
@@ -655,9 +681,9 @@ affile_dd_open_writer(gpointer args[])
           else
             {
               log_pipe_ref(&next->super);
-              g_static_mutex_lock(&self->lock);
+              g_mutex_lock(&self->lock);
               g_hash_table_insert(self->writer_hash, next->filename, next);
-              g_static_mutex_unlock(&self->lock);
+              g_mutex_unlock(&self->lock);
             }
         }
       else
@@ -688,10 +714,10 @@ affile_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
       /* we need to lock single_writer in order to get a reference and
        * make sure it is not a stale pointer by the time we ref it */
 
-      g_static_mutex_lock(&self->lock);
+      g_mutex_lock(&self->lock);
       if (!self->single_writer)
         {
-          g_static_mutex_unlock(&self->lock);
+          g_mutex_unlock(&self->lock);
           next = main_loop_call((void *(*)(void *)) affile_dd_open_writer, args, TRUE);
         }
       else
@@ -699,7 +725,7 @@ affile_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
           next = self->single_writer;
           next->queue_pending = TRUE;
           log_pipe_ref(&next->super);
-          g_static_mutex_unlock(&self->lock);
+          g_mutex_unlock(&self->lock);
         }
     }
   else
@@ -707,10 +733,10 @@ affile_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
       GString *filename;
 
       filename = g_string_sized_new(32);
-      LogTemplateEvalOptions options = {&self->writer_options.template_options, LTZ_LOCAL, 0, NULL};
+      LogTemplateEvalOptions options = {&self->writer_options.template_options, LTZ_LOCAL, 0, NULL, LM_VT_STRING};
       log_template_format(self->filename_template, msg, &options, filename);
 
-      g_static_mutex_lock(&self->lock);
+      g_mutex_lock(&self->lock);
       if (self->writer_hash)
         next = g_hash_table_lookup(self->writer_hash, filename->str);
       else
@@ -720,11 +746,11 @@ affile_dd_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
         {
           log_pipe_ref(&next->super);
           next->queue_pending = TRUE;
-          g_static_mutex_unlock(&self->lock);
+          g_mutex_unlock(&self->lock);
         }
       else
         {
-          g_static_mutex_unlock(&self->lock);
+          g_mutex_unlock(&self->lock);
           args[1] = filename;
           next = main_loop_call((void *(*)(void *)) affile_dd_open_writer, args, TRUE);
         }
@@ -746,7 +772,7 @@ affile_dd_free(LogPipe *s)
 {
   AFFileDestDriver *self = (AFFileDestDriver *) s;
 
-  g_static_mutex_free(&self->lock);
+  g_mutex_clear(&self->lock);
   affile_dest_drivers = g_list_remove(affile_dest_drivers, self);
 
   /* NOTE: this must be NULL as deinit has freed it, otherwise we'd have circular references */
@@ -756,6 +782,7 @@ affile_dd_free(LogPipe *s)
   log_writer_options_destroy(&self->writer_options);
   file_opener_options_deinit(&self->file_opener_options);
   file_opener_free(self->file_opener);
+  g_free(self->symlink_as);
   log_dest_driver_free(s);
 }
 
@@ -783,7 +810,7 @@ affile_dd_new_instance(LogTemplate *filename_template, GlobalConfig *cfg)
   file_opener_options_defaults(&self->file_opener_options);
 
   affile_dd_set_time_reap(&self->super.super, self->filename_is_a_template ? -1 : 0);
-  g_static_mutex_init(&self->lock);
+  g_mutex_init(&self->lock);
 
   affile_dest_drivers = g_list_append(affile_dest_drivers, self);
 
