@@ -95,21 +95,28 @@ _exec_program(const gchar *cmdline)
 static void
 _close_all_fd(void)
 {
+  const rlim_t min_range = 10000;
   struct rlimit rlp;
   if (getrlimit(RLIMIT_NOFILE, &rlp) < 0)
     {
       /*
        * Failing to query max *fd*, still closing an arbitrary range might make sense,
-       * tring with some arbitrary big number. The only issue could be that it cannot
+       * trying with some arbitrary big number. The only issue could be that it cannot
        * close all of the needed *fd* and fail to bind to a socket (original issue).
        */
-      rlp.rlim_max = 10000;
+      rlp.rlim_max = min_range;
+    }
+  else if (rlp.rlim_max == RLIM_INFINITY)
+    {
+      /*
+       * Also could happen that the limits are RLIM_INFINITY.
+       * Use the same logic as above, try to close as much as possible.
+       */
+      rlp.rlim_max = (rlp.rlim_cur != RLIM_INFINITY ? rlp.rlim_cur : min_range);
     }
 
-  for (int i = rlp.rlim_max; i > 2; --i)
-    {
-      close(i);
-    }
+  for (rlim_t i = rlp.rlim_max; i > 2; --i)
+    close(i);
 }
 
 static void
@@ -283,6 +290,7 @@ afprogram_sd_init(LogPipe *s)
       proto = log_proto_text_server_new(transport, &self->reader_options.proto_options.super);
 
       self->reader = log_reader_new(s->cfg);
+      log_pipe_set_options(&self->reader->super.super, &self->super.super.super.options);
       log_reader_open(self->reader, proto, poll_fd_events_new(fd));
       log_reader_set_options(self->reader,
                              s,
@@ -529,13 +537,25 @@ afprogram_dd_init(LogPipe *s)
   if (!self->writer)
     self->writer = log_writer_new(LW_FORMAT_FILE, s->cfg);
 
+  log_pipe_set_options((LogPipe *) self->writer, &self->super.super.super.options);
   log_writer_set_options(self->writer,
                          s,
                          &self->writer_options,
                          self->super.super.id,
                          self->process_info.cmdline->str);
-  log_writer_set_queue(self->writer, log_dest_driver_acquire_queue(&self->super,
-                       afprogram_dd_format_queue_persist_name(self)));
+
+  StatsClusterKeyBuilder *driver_sck_builder = stats_cluster_key_builder_new();
+  StatsClusterKeyBuilder *queue_sck_builder = stats_cluster_key_builder_new();
+  log_writer_init_driver_sck_builder(self->writer, driver_sck_builder);
+  log_writer_init_queue_sck_builder(self->writer, queue_sck_builder);
+
+  gint stats_level = log_pipe_is_internal(&self->super.super.super) ? STATS_LEVEL3 : self->writer_options.stats_level;
+  LogQueue *queue = log_dest_driver_acquire_queue(&self->super, afprogram_dd_format_queue_persist_name(self),
+                                                  stats_level, driver_sck_builder, queue_sck_builder);
+  log_writer_set_queue(self->writer, queue);
+
+  stats_cluster_key_builder_free(queue_sck_builder);
+  stats_cluster_key_builder_free(driver_sck_builder);
 
   if (!log_pipe_init((LogPipe *) self->writer))
     {
@@ -563,7 +583,7 @@ afprogram_dd_store_reload_store_item(AFProgramDestDriver *self, GlobalConfig *cf
   reload_info->writer = self->writer;
 
   cfg_persist_config_add(cfg, afprogram_dd_format_persist_name((const LogPipe *)self), reload_info,
-                         afprogram_reload_store_item_destroy_notify, FALSE);
+                         afprogram_reload_store_item_destroy_notify);
 }
 
 static gboolean
