@@ -409,7 +409,7 @@ snmpdest_worker_insert(LogThreadedDestDriver *s, LogMessage *msg)
   GList *snmp_template = self->snmp_templates;
   GString *fs = g_string_sized_new(128);
 
-  LogTemplateEvalOptions options = {&self->template_options, LTZ_LOCAL, 0, NULL};
+  LogTemplateEvalOptions options = {&self->template_options, LTZ_LOCAL, 0, NULL, LM_VT_STRING};
   /* go through the snmp objects and add them to the message */
   while (snmp_obj)
     {
@@ -624,9 +624,6 @@ snmpdest_dd_init(LogPipe *s)
   SNMPDestDriver *self = (SNMPDestDriver *)s;
   GlobalConfig *cfg = log_pipe_get_config(s);
 
-  if (!log_threaded_dest_driver_init_method(s))
-    return FALSE;
-
   msg_verbose("Initializing SNMP destination",
               evt_tag_str("driver", self->super.super.super.id),
               evt_tag_str("host", self->host),
@@ -639,6 +636,9 @@ snmpdest_dd_init(LogPipe *s)
       return FALSE;
     }
 
+  if (!log_threaded_dest_driver_init_method(s))
+    return FALSE;
+
   log_template_options_init(&self->template_options, cfg);
   return TRUE;
 }
@@ -759,7 +759,6 @@ snmpdest_dd_new(GlobalConfig *cfg)
   self->transport = g_strdup("UDP");
 
   log_template_options_defaults(&self->template_options);
-  self->worker_options.is_output_thread = TRUE;
 
   return (LogDriver *)self;
 }
@@ -810,6 +809,14 @@ gchar *snmpdest_dd_get_version(LogDriver *d)
   return self->version;
 }
 
+const LogTemplateOptions *
+snmpdest_dd_get_template_options(LogDriver *d)
+{
+  SNMPDestDriver *self = (SNMPDestDriver *) d;
+
+  return &self->template_options;
+}
+
 gboolean snmpdest_dd_check_auth_algorithm(gchar *algo)
 {
   if (strcasecmp(algo, s_sha) == 0)
