@@ -485,7 +485,10 @@ afsocket_dd_construct_writer_method(AFSocketDestDriver *self)
   if (self->transport_mapper->sock_type == SOCK_STREAM && self->close_on_input)
     writer_flags |= LW_DETECT_EOF;
 
-  return log_writer_new(writer_flags, self->super.super.super.cfg);
+  LogWriter *writer = log_writer_new(writer_flags, self->super.super.super.cfg);
+  log_pipe_set_options((LogPipe *) writer, &self->super.super.super.options);
+
+  return writer;
 }
 
 static gboolean
@@ -505,8 +508,19 @@ afsocket_dd_setup_writer(AFSocketDestDriver *self)
                          &self->writer_options,
                          self->super.super.id,
                          afsocket_dd_stats_instance(self));
-  log_writer_set_queue(self->writer, log_dest_driver_acquire_queue(
-                         &self->super, afsocket_dd_format_qfile_name(self)));
+
+  StatsClusterKeyBuilder *driver_sck_builder = stats_cluster_key_builder_new();
+  StatsClusterKeyBuilder *queue_sck_builder = stats_cluster_key_builder_new();
+  log_writer_init_driver_sck_builder(self->writer, driver_sck_builder);
+  log_writer_init_queue_sck_builder(self->writer, queue_sck_builder);
+
+  gint stats_level = log_pipe_is_internal(&self->super.super.super) ? STATS_LEVEL3 : self->writer_options.stats_level;
+  LogQueue *queue = log_dest_driver_acquire_queue(&self->super, afsocket_dd_format_qfile_name(self),
+                                                  stats_level, driver_sck_builder, queue_sck_builder);
+  log_writer_set_queue(self->writer, queue);
+
+  stats_cluster_key_builder_free(queue_sck_builder);
+  stats_cluster_key_builder_free(driver_sck_builder);
 
   if (!log_pipe_init((LogPipe *) self->writer))
     {
@@ -625,7 +639,7 @@ afsocket_dd_save_connection(AFSocketDestDriver *self)
     {
       ReloadStoreItem *item = _reload_store_item_new(self);
       cfg_persist_config_add(cfg, afsocket_dd_format_connections_name(self), item,
-                             (GDestroyNotify)_reload_store_item_free, FALSE);
+                             (GDestroyNotify)_reload_store_item_free);
       self->writer = NULL;
     }
 }
