@@ -123,6 +123,7 @@ extern int correlation_debug;
     KW_LIFETIME = 10403,           /* KW_LIFETIME  */
     KW_MAX_DYNAMIC = 10404,        /* KW_MAX_DYNAMIC  */
     KW_SYSLOG_STATS = 10405,       /* KW_SYSLOG_STATS  */
+    KW_HEALTHCHECK_FREQ = 10406,   /* KW_HEALTHCHECK_FREQ  */
     KW_CHAIN_HOSTNAMES = 10090,    /* KW_CHAIN_HOSTNAMES  */
     KW_NORMALIZE_HOSTNAMES = 10091, /* KW_NORMALIZE_HOSTNAMES  */
     KW_KEEP_HOSTNAME = 10092,      /* KW_KEEP_HOSTNAME  */
@@ -163,7 +164,10 @@ extern int correlation_debug;
     KW_SEND_TIME_ZONE = 10203,     /* KW_SEND_TIME_ZONE  */
     KW_LOCAL_TIME_ZONE = 10204,    /* KW_LOCAL_TIME_ZONE  */
     KW_FORMAT = 10205,             /* KW_FORMAT  */
-    KW_TRUNCATE_SIZE = 10206,      /* KW_TRUNCATE_SIZE  */
+    KW_MULTI_LINE_MODE = 10206,    /* KW_MULTI_LINE_MODE  */
+    KW_MULTI_LINE_PREFIX = 10207,  /* KW_MULTI_LINE_PREFIX  */
+    KW_MULTI_LINE_GARBAGE = 10208, /* KW_MULTI_LINE_GARBAGE  */
+    KW_TRUNCATE_SIZE = 10209,      /* KW_TRUNCATE_SIZE  */
     KW_TIME_REOPEN = 10210,        /* KW_TIME_REOPEN  */
     KW_TIME_REAP = 10211,          /* KW_TIME_REAP  */
     KW_TIME_SLEEP = 10212,         /* KW_TIME_SLEEP  */
@@ -224,6 +228,8 @@ extern int correlation_debug;
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
     KW_CAST = 10511,               /* KW_CAST  */
+    KW_UPPER = 10512,              /* KW_UPPER  */
+    KW_LOWER = 10513,              /* KW_LOWER  */
     KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
     KW_RETRIES = 10521,            /* KW_RETRIES  */
     KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
@@ -241,7 +247,9 @@ extern int correlation_debug;
     KW_PROGRAM_TEMPLATE = 10535,   /* KW_PROGRAM_TEMPLATE  */
     KW_MESSAGE_TEMPLATE = 10536,   /* KW_MESSAGE_TEMPLATE  */
     KW_SORT_KEY = 10537,           /* KW_SORT_KEY  */
-    KW_PREFIX = 10538              /* KW_PREFIX  */
+    KW_PREFIX = 10538,             /* KW_PREFIX  */
+    KW_GROUP_LINES = 10539,        /* KW_GROUP_LINES  */
+    KW_LINE_SEPARATOR = 10540      /* KW_LINE_SEPARATOR  */
   };
   typedef enum correlation_tokentype correlation_token_kind_t;
 #endif
@@ -311,6 +319,7 @@ extern int correlation_debug;
 #define KW_LIFETIME 10403
 #define KW_MAX_DYNAMIC 10404
 #define KW_SYSLOG_STATS 10405
+#define KW_HEALTHCHECK_FREQ 10406
 #define KW_CHAIN_HOSTNAMES 10090
 #define KW_NORMALIZE_HOSTNAMES 10091
 #define KW_KEEP_HOSTNAME 10092
@@ -351,7 +360,10 @@ extern int correlation_debug;
 #define KW_SEND_TIME_ZONE 10203
 #define KW_LOCAL_TIME_ZONE 10204
 #define KW_FORMAT 10205
-#define KW_TRUNCATE_SIZE 10206
+#define KW_MULTI_LINE_MODE 10206
+#define KW_MULTI_LINE_PREFIX 10207
+#define KW_MULTI_LINE_GARBAGE 10208
+#define KW_TRUNCATE_SIZE 10209
 #define KW_TIME_REOPEN 10210
 #define KW_TIME_REAP 10211
 #define KW_TIME_SLEEP 10212
@@ -412,6 +424,8 @@ extern int correlation_debug;
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
 #define KW_CAST 10511
+#define KW_UPPER 10512
+#define KW_LOWER 10513
 #define KW_ON_ERROR 10520
 #define KW_RETRIES 10521
 #define KW_FETCH_NO_DATA_DELAY 10522
@@ -430,6 +444,8 @@ extern int correlation_debug;
 #define KW_MESSAGE_TEMPLATE 10536
 #define KW_SORT_KEY 10537
 #define KW_PREFIX 10538
+#define KW_GROUP_LINES 10539
+#define KW_LINE_SEPARATOR 10540
 
 /* Value type.  */
 #if ! defined CORRELATION_STYPE && ! defined CORRELATION_STYPE_IS_DECLARED
