@@ -25,6 +25,7 @@
 
 #include "diskq.h"
 #include "diskq-config.h"
+#include "diskq-global-metrics.h"
 
 #include "driver.h"
 #include "messages.h"
@@ -52,67 +53,123 @@ log_queue_disk_is_file_in_directory(const gchar *file, const gchar *directory)
 }
 
 static LogQueue *
-_acquire_queue(LogDestDriver *dd, const gchar *persist_name)
+_create_disk_queue(DiskQDestPlugin *self, const gchar *filename, const gchar *persist_name, gint stats_level,
+                   const StatsClusterKeyBuilder *driver_sck_builder, StatsClusterKeyBuilder *queue_sck_builder)
 {
-  DiskQDestPlugin *self = log_driver_get_plugin(&dd->super, DiskQDestPlugin, DISKQ_PLUGIN_NAME);
-  GlobalConfig *cfg = log_pipe_get_config(&dd->super.super);
-  LogQueue *queue = NULL;
-  gchar *qfile_name;
-  gboolean success;
-
-  if (persist_name)
-    queue = cfg_persist_config_fetch(cfg, persist_name);
-
-  if (queue)
-    {
-      log_queue_unref(queue);
-      queue = NULL;
-    }
-
   if (self->options.reliable)
-    queue = log_queue_disk_reliable_new(&self->options, persist_name);
-  else
-    queue = log_queue_disk_non_reliable_new(&self->options, persist_name);
-  log_queue_set_throttle(queue, dd->throttle);
+    return log_queue_disk_reliable_new(&self->options, filename, persist_name, stats_level, driver_sck_builder,
+                                       queue_sck_builder);
 
-  qfile_name = persist_state_lookup_string(cfg->state, persist_name, NULL, NULL);
+  return log_queue_disk_non_reliable_new(&self->options, filename, persist_name, stats_level, driver_sck_builder,
+                                         queue_sck_builder);
+}
 
-  if (qfile_name && !log_queue_disk_is_file_in_directory(qfile_name, self->options.dir))
+static void
+_warn_if_dir_changed(const gchar *qfile_name, const gchar *dir)
+{
+  if (!log_queue_disk_is_file_in_directory(qfile_name, dir))
     {
       msg_warning("The disk buffer directory has changed in the configuration, but the disk queue file cannot be moved",
                   evt_tag_str("qfile", qfile_name),
-                  evt_tag_str("dir", self->options.dir));
+                  evt_tag_str("dir", dir));
     }
+}
 
-  success = log_queue_disk_load_queue(queue, qfile_name);
-  if (!success)
+static LogQueue *
+_create_and_start_disk_queue_with_filename_from_persist(DiskQDestPlugin *self, const gchar *persist_qfile_name,
+                                                        const gchar *persist_name, gint stats_level,
+                                                        const StatsClusterKeyBuilder *driver_sck_builder,
+                                                        StatsClusterKeyBuilder *queue_sck_builder)
+{
+  if (!persist_qfile_name)
+    return FALSE;
+
+  _warn_if_dir_changed(persist_qfile_name, self->options.dir);
+
+  LogQueue *queue = _create_disk_queue(self, persist_qfile_name, persist_name, stats_level, driver_sck_builder,
+                                       queue_sck_builder);
+  if (log_queue_disk_start(queue))
+    return queue;
+
+  log_queue_unref(queue);
+
+  gchar *new_qfile_name = qdisk_get_next_filename(self->options.dir, self->options.reliable);
+  if (!new_qfile_name)
+    return NULL;
+
+  queue = _create_disk_queue(self, persist_qfile_name, persist_name, stats_level, driver_sck_builder, queue_sck_builder);
+  if (log_queue_disk_start(queue))
     {
-      if (qfile_name && log_queue_disk_load_queue(queue, NULL))
-        {
-          msg_error("Error opening disk-queue file, a new one started",
-                    evt_tag_str("old_filename", qfile_name),
-                    evt_tag_str("new_filename", log_queue_disk_get_filename(queue)));
-        }
-      else
-        {
-          g_free(qfile_name);
-          msg_error("Error initializing log queue");
-          return NULL;
-        }
+      msg_error("Error opening disk-queue file, a new one started",
+                evt_tag_str("old_filename", persist_qfile_name),
+                evt_tag_str("new_filename", log_queue_disk_get_filename(queue)));
+      g_free(new_qfile_name);
+      return queue;
     }
 
-  g_free(qfile_name);
+  msg_error("Error initializing log queue");
+
+  log_queue_unref(queue);
+  g_free(new_qfile_name);
+  return NULL;
+}
+
+static LogQueue *
+_create_and_start_disk_queue_with_new_filename(DiskQDestPlugin *self, const gchar *new_qfile_name,
+                                               const gchar *persist_name, gint stats_level,
+                                               const StatsClusterKeyBuilder *driver_sck_builder,
+                                               StatsClusterKeyBuilder *queue_sck_builder)
+{
+  if (!new_qfile_name)
+    return NULL;
+
+  LogQueue *queue = _create_disk_queue(self, new_qfile_name, persist_name, stats_level, driver_sck_builder,
+                                       queue_sck_builder);
+  if (log_queue_disk_start(queue))
+    return queue;
+
+  msg_error("Error initializing log queue");
+
+  log_queue_unref(queue);
+  return NULL;
+}
+
+static LogQueue *
+_acquire_queue(LogDestDriver *dd, const gchar *persist_name, gint stats_level,
+               const StatsClusterKeyBuilder *driver_sck_builder, StatsClusterKeyBuilder *queue_sck_builder)
+{
+  DiskQDestPlugin *self = log_driver_get_plugin(&dd->super, DiskQDestPlugin, DISKQ_PLUGIN_NAME);
+  GlobalConfig *cfg = log_pipe_get_config(&dd->super.super);
+  LogQueue *queue;
+  gchar *persist_qfile_name, *new_qfile_name = NULL;
 
   if (persist_name)
+    log_queue_unref(cfg_persist_config_fetch(cfg, persist_name));
+
+  persist_qfile_name = persist_state_lookup_string(cfg->state, persist_name, NULL, NULL);
+  queue = _create_and_start_disk_queue_with_filename_from_persist(
+            self, persist_qfile_name, persist_name, stats_level, driver_sck_builder, queue_sck_builder);
+  if (queue)
+    goto exit;
+
+  new_qfile_name = qdisk_get_next_filename(self->options.dir, self->options.reliable);
+  queue = _create_and_start_disk_queue_with_new_filename(self, new_qfile_name, persist_name, stats_level,
+                                                         driver_sck_builder, queue_sck_builder);
+
+exit:
+  if (queue)
     {
-      /* save the queue file name to permanent state */
-      qfile_name = (gchar *) log_queue_disk_get_filename(queue);
-      if (qfile_name)
-        {
-          persist_state_alloc_string(cfg->state, persist_name, qfile_name, -1);
-        }
+      log_queue_set_throttle(queue, dd->throttle);
+
+      const gchar *qfile_name = log_queue_disk_get_filename(queue);
+      diskq_global_metrics_file_acquired(qfile_name);
+      if (persist_name && qfile_name)
+        persist_state_alloc_string(cfg->state, persist_name, qfile_name, -1);
     }
 
+  g_free(persist_qfile_name);
+  g_free(new_qfile_name);
+
   return queue;
 }
 
@@ -122,10 +179,12 @@ _release_queue(LogDestDriver *dd, LogQueue *queue)
   GlobalConfig *cfg = log_pipe_get_config(&dd->super.super);
   gboolean persistent;
 
-  log_queue_disk_save_queue(queue, &persistent);
+  log_queue_disk_stop(queue, &persistent);
+  diskq_global_metrics_file_released(log_queue_disk_get_filename(queue));
+
   if (queue->persist_name)
     {
-      cfg_persist_config_add(cfg, queue->persist_name, queue, (GDestroyNotify) log_queue_unref, FALSE);
+      cfg_persist_config_add(cfg, queue->persist_name, queue, (GDestroyNotify) log_queue_unref);
     }
   else
     {
