@@ -124,16 +124,16 @@ open_queue(char *filename, LogQueue **lq, DiskQueueOptions *options)
   if (options->reliable)
     {
       options->mem_buf_size = 1024 * 1024;
-      *lq = log_queue_disk_reliable_new(options, NULL);
+      *lq = log_queue_disk_reliable_new(options, filename, NULL, STATS_LEVEL0, NULL, NULL);
     }
   else
     {
       options->mem_buf_size = 128;
       options->qout_size = 1000;
-      *lq = log_queue_disk_non_reliable_new(options, NULL);
+      *lq = log_queue_disk_non_reliable_new(options, filename, NULL, STATS_LEVEL0, NULL, NULL);
     }
 
-  if (!log_queue_disk_load_queue(*lq, filename))
+  if (!log_queue_disk_start(*lq))
     {
       fprintf(stderr, "Error restoring disk buffer file.\n");
       return FALSE;
@@ -195,6 +195,8 @@ dqtool_cat(int argc, char *argv[])
           printf("%s", msg->str);
         }
 
+      gboolean persistent;
+      log_queue_disk_stop(lq, &persistent);
       log_queue_unref(lq);
     }
   g_string_free(msg, TRUE);
@@ -214,6 +216,9 @@ dqtool_info(int argc, char *argv[])
 
       if (!open_queue(argv[i], &lq, &options))
         continue;
+
+      gboolean persistent;
+      log_queue_disk_stop(lq, &persistent);
       log_queue_unref(lq);
     }
   return 0;
