@@ -29,51 +29,34 @@
 
 #define ITEM_NUMBER_PER_MESSAGE 2
 
-typedef struct
-{
-  guint index_in_queue;
-  guint item_number_per_message;
-  LogQueue *queue;
-} DiskqMemusageLoaderState;
-
-static gboolean
-_object_is_message_in_position(guint index_in_queue, guint item_number_per_message)
-{
-  return !(index_in_queue % item_number_per_message);
-}
-
 static void
-_update_memory_usage_during_load(gpointer data, gpointer s)
+_update_memory_usage_during_load(LogQueueDiskNonReliable *s, GQueue *memory_queue, guint offset)
 {
-  DiskqMemusageLoaderState *state = (DiskqMemusageLoaderState *)s;
+  if (g_queue_get_length(memory_queue) == offset)
+    return;
 
-  if (_object_is_message_in_position(state->index_in_queue, state->item_number_per_message))
+  GList *first_link = g_queue_peek_nth_link(memory_queue, offset);
+  for (GList *msg_link = first_link; msg_link; msg_link = msg_link->next->next)
     {
-      LogMessage *msg = (LogMessage *)data;
-      log_queue_memory_usage_add(state->queue, log_msg_get_size(msg));
+      LogMessage *msg = (LogMessage *) msg_link->data;
+      log_queue_memory_usage_add(&s->super.super, log_msg_get_size(msg));
     }
-  state->index_in_queue++;
 }
 
 static gboolean
-_start(LogQueueDisk *s, const gchar *filename)
+_start(LogQueueDisk *s)
 {
   LogQueueDiskNonReliable *self = (LogQueueDiskNonReliable *) s;
 
-  gboolean retval = qdisk_start(s->qdisk, filename, self->qout, self->qbacklog, self->qoverflow);
+  guint qout_length_before_start = g_queue_get_length(self->qout);
+  guint qbacklog_length_before_start = g_queue_get_length(self->qbacklog);
+  guint qoverflow_length_before_start = g_queue_get_length(self->qoverflow);
 
-  DiskqMemusageLoaderState qout_sum = { .index_in_queue = 0,
-                                        .item_number_per_message = ITEM_NUMBER_PER_MESSAGE,
-                                        .queue = &self->super.super
-                                      };
+  gboolean retval = qdisk_start(s->qdisk, self->qout, self->qbacklog, self->qoverflow);
 
-  DiskqMemusageLoaderState overflow_sum = { .index_in_queue = 0,
-                                            .item_number_per_message = ITEM_NUMBER_PER_MESSAGE,
-                                            .queue = &self->super.super
-                                          };
-
-  g_queue_foreach(self->qout, _update_memory_usage_during_load, &qout_sum);
-  g_queue_foreach(self->qoverflow, _update_memory_usage_during_load, &overflow_sum);
+  _update_memory_usage_during_load(self, self->qout, qout_length_before_start);
+  _update_memory_usage_during_load(self, self->qbacklog, qbacklog_length_before_start);
+  _update_memory_usage_during_load(self, self->qoverflow, qoverflow_length_before_start);
 
   return retval;
 }
@@ -117,7 +100,7 @@ _serialize_and_write_message_to_disk(LogQueueDiskNonReliable *self, LogMessage *
 {
   ScratchBuffersMarker marker;
   GString *write_serialized = scratch_buffers_alloc_and_mark(&marker);
-  if (!qdisk_serialize_msg(self->super.qdisk, msg, write_serialized))
+  if (!log_queue_disk_serialize_msg(&self->super, msg, write_serialized))
     {
       scratch_buffers_reclaim_marked(marker);
       return FALSE;
@@ -151,6 +134,7 @@ _move_messages_from_overflow(LogQueueDiskNonReliable *self)
         {
           if (_serialize_and_write_message_to_disk(self, msg))
             {
+              log_queue_disk_update_disk_related_counters(&self->super);
               log_queue_memory_usage_sub(&self->super.super, log_msg_get_size(msg));
               log_msg_ack(msg, &path_options, AT_PROCESSED);
               log_msg_unref(msg);
@@ -169,7 +153,7 @@ _move_messages_from_overflow(LogQueueDiskNonReliable *self)
     }
 }
 
-static void
+static gboolean
 _move_messages_from_disk_to_qout(LogQueueDiskNonReliable *self)
 {
   do
@@ -181,26 +165,33 @@ _move_messages_from_disk_to_qout(LogQueueDiskNonReliable *self)
       LogMessage *msg = log_queue_disk_read_message(&self->super, &path_options);
 
       if (!msg)
-        break;
+        return FALSE;
 
       g_queue_push_tail(self->qout, msg);
       g_queue_push_tail(self->qout, LOG_PATH_OPTIONS_TO_POINTER(&path_options));
       log_queue_memory_usage_add(&self->super.super, log_msg_get_size(msg));
+      log_queue_disk_update_disk_related_counters(&self->super);
     }
   while (HAS_SPACE_IN_QUEUE(self->qout));
+
+  return TRUE;
 }
 
-static inline void
+static inline gboolean
 _maybe_move_messages_among_queue_segments(LogQueueDiskNonReliable *self)
 {
+  gboolean ret = TRUE;
+
   if (qdisk_is_read_only(self->super.qdisk))
-    return;
+    return TRUE;
 
   if (self->qout->length == 0 && self->qout_size > 0)
-    _move_messages_from_disk_to_qout(self);
+    ret = _move_messages_from_disk_to_qout(self);
 
   if (self->qoverflow->length > 0)
     _move_messages_from_overflow(self);
+
+  return ret;
 }
 
 /* runs only in the output thread */
@@ -218,6 +209,7 @@ _ack_backlog(LogQueue *s, gint num_msg_to_ack)
         return;
       msg = g_queue_pop_head(self->qbacklog);
       POINTER_TO_LOG_PATH_OPTIONS(g_queue_pop_head(self->qbacklog), &path_options);
+      log_queue_memory_usage_sub(&self->super.super, log_msg_get_size(msg));
       log_msg_ack(msg, &path_options, AT_PROCESSED);
       log_msg_unref(msg);
     }
@@ -229,7 +221,7 @@ _rewind_backlog(LogQueue *s, guint rewind_count)
   guint i;
   LogQueueDiskNonReliable *self = (LogQueueDiskNonReliable *)s;
 
-  g_static_mutex_lock(&s->lock);
+  g_mutex_lock(&s->lock);
 
   rewind_count = MIN(rewind_count, _get_message_number_in_queue(self->qbacklog));
 
@@ -242,16 +234,15 @@ _rewind_backlog(LogQueue *s, guint rewind_count)
       g_queue_push_head(self->qout, ptr_msg);
 
       log_queue_queued_messages_inc(s);
-      log_queue_memory_usage_add(s, log_msg_get_size((LogMessage *)ptr_msg));
     }
 
-  g_static_mutex_unlock(&s->lock);
+  g_mutex_unlock(&s->lock);
 }
 
 static void
 _rewind_backlog_all(LogQueue *s)
 {
-  _rewind_backlog(s, -1);
+  _rewind_backlog(s, G_MAXUINT);
 }
 
 static inline LogMessage *
@@ -281,6 +272,7 @@ _push_tail_qbacklog(LogQueueDiskNonReliable *self, LogMessage *msg, LogPathOptio
   log_msg_ref(msg);
   g_queue_push_tail(self->qbacklog, msg);
   g_queue_push_tail(self->qbacklog, LOG_PATH_OPTIONS_TO_POINTER(path_options));
+  log_queue_memory_usage_add(&self->super.super, log_msg_get_size(msg));
 }
 
 static LogMessage *
@@ -288,8 +280,9 @@ _pop_head(LogQueue *s, LogPathOptions *path_options)
 {
   LogQueueDiskNonReliable *self = (LogQueueDiskNonReliable *)s;
   LogMessage *msg = NULL;
+  gboolean stats_update = TRUE;
 
-  g_static_mutex_lock(&s->lock);
+  g_mutex_lock(&s->lock);
 
   if (self->qout->length > 0)
     {
@@ -307,19 +300,24 @@ _pop_head(LogQueue *s, LogPathOptions *path_options)
 
   if (!msg)
     {
-      g_static_mutex_unlock(&s->lock);
+      g_mutex_unlock(&s->lock);
       return NULL;
     }
 
 success:
-  _maybe_move_messages_among_queue_segments(self);
+  if (!_maybe_move_messages_among_queue_segments(self))
+    {
+      stats_update = FALSE;
+    }
 
-  g_static_mutex_unlock(&s->lock);
+  log_queue_disk_update_disk_related_counters(&self->super);
+  g_mutex_unlock(&s->lock);
 
   if (s->use_backlog)
     _push_tail_qbacklog(self, msg, path_options);
 
-  log_queue_queued_messages_dec(s);
+  if (stats_update)
+    log_queue_queued_messages_dec(s);
 
   return msg;
 }
@@ -336,7 +334,7 @@ _push_head(LogQueue *s, LogMessage *msg, const LogPathOptions *path_options)
 static inline gboolean
 _is_msg_serialization_needed_hint(LogQueueDiskNonReliable *self)
 {
-  g_static_mutex_lock(&self->super.super.lock);
+  g_mutex_lock(&self->super.super.lock);
 
   gboolean msg_serialization_needed = FALSE;
 
@@ -352,7 +350,7 @@ _is_msg_serialization_needed_hint(LogQueueDiskNonReliable *self)
   msg_serialization_needed = TRUE;
 
 exit:
-  g_static_mutex_unlock(&self->super.super.lock);
+  g_mutex_unlock(&self->super.super.lock);
   return msg_serialization_needed;
 }
 
@@ -394,12 +392,16 @@ static inline gboolean
 _push_tail_disk(LogQueueDiskNonReliable *self, LogMessage *msg, const LogPathOptions *path_options,
                 GString *serialized_msg)
 {
-  if (!_ensure_serialized_and_write_to_disk(self, msg, serialized_msg))
-    return FALSE;
+  gboolean result = _ensure_serialized_and_write_to_disk(self, msg, serialized_msg);
+  if (result)
+    {
+      log_msg_ack(msg, path_options, AT_PROCESSED);
+      log_msg_unref(msg);
+    }
 
-  log_msg_ack(msg, path_options, AT_PROCESSED);
-  log_msg_unref(msg);
-  return TRUE;
+  log_queue_disk_update_disk_related_counters(&self->super);
+
+  return result;
 }
 
 static void
@@ -413,7 +415,7 @@ _push_tail(LogQueue *s, LogMessage *msg, const LogPathOptions *path_options)
   if (_is_msg_serialization_needed_hint(self))
     {
       serialized_msg = scratch_buffers_alloc_and_mark(&marker);
-      if (!qdisk_serialize_msg(self->super.qdisk, msg, serialized_msg))
+      if (!log_queue_disk_serialize_msg(&self->super, msg, serialized_msg))
         {
           msg_error("Failed to serialize message for non-reliable disk-buffer, dropping message",
                     evt_tag_str("filename", qdisk_get_filename(self->super.qdisk)),
@@ -424,7 +426,7 @@ _push_tail(LogQueue *s, LogMessage *msg, const LogPathOptions *path_options)
         }
     }
 
-  g_static_mutex_lock(&s->lock);
+  g_mutex_lock(&s->lock);
 
   /* we push messages into queue segments in the following order: qoverflow, disk, qout */
   if (_can_push_to_qout(self))
@@ -452,29 +454,35 @@ _push_tail(LogQueue *s, LogMessage *msg, const LogPathOptions *path_options)
     }
 
 queued:
-  log_queue_push_notify(s);
   log_queue_queued_messages_inc(s);
 
+  /* this releases the queue's lock for a short time, which may violate the
+   * consistency of the disk-buffer, so it must be the last call under lock in this function
+   */
+  log_queue_push_notify(s);
+
 exit:
-  g_static_mutex_unlock(&s->lock);
+  g_mutex_unlock(&s->lock);
   if (serialized_msg)
     scratch_buffers_reclaim_marked(marker);
 }
 
 static void
-_free_queue(GQueue *q)
+_empty_queue(LogQueueDiskNonReliable *self, GQueue *q)
 {
-  while (!g_queue_is_empty(q))
+  while (q && !g_queue_is_empty(q))
     {
       LogMessage *lm;
       LogPathOptions path_options = LOG_PATH_OPTIONS_INIT;
 
       lm = g_queue_pop_head(q);
       POINTER_TO_LOG_PATH_OPTIONS(g_queue_pop_head(q), &path_options);
+
+      log_queue_memory_usage_sub(&self->super.super, log_msg_get_size(lm));
+
       log_msg_ack(lm, &path_options, AT_PROCESSED);
       log_msg_unref(lm);
     }
-  g_queue_free(q);
 }
 
 static void
@@ -482,44 +490,55 @@ _free(LogQueue *s)
 {
   LogQueueDiskNonReliable *self = (LogQueueDiskNonReliable *)s;
 
-  _free_queue(self->qoverflow);
-  self->qoverflow = NULL;
-  _free_queue(self->qout);
-  self->qout = NULL;
-  _free_queue(self->qbacklog);
-  self->qbacklog = NULL;
+
+  if (self->qout)
+    {
+      g_assert(g_queue_is_empty(self->qout));
+      g_queue_free(self->qout);
+      self->qout = NULL;
+    }
+
+  if (self->qbacklog)
+    {
+      g_assert(g_queue_is_empty(self->qbacklog));
+      g_queue_free(self->qbacklog);
+      self->qbacklog = NULL;
+    }
+
+  if (self->qoverflow)
+    {
+      g_assert(g_queue_is_empty(self->qoverflow));
+      g_queue_free(self->qoverflow);
+      self->qoverflow = NULL;
+    }
 
   log_queue_disk_free_method(&self->super);
 }
 
 static gboolean
-_load_queue(LogQueueDisk *s, const gchar *filename)
+_stop(LogQueueDisk *s, gboolean *persistent)
 {
-  /* qdisk portion is not yet started when this happens */
-  g_assert(!qdisk_started(s->qdisk));
+  LogQueueDiskNonReliable *self = (LogQueueDiskNonReliable *) s;
 
-  return _start(s, filename);
+  gboolean result = FALSE;
+
+  if (qdisk_stop(s->qdisk, self->qout, self->qbacklog, self->qoverflow))
+    {
+      *persistent = TRUE;
+      result = TRUE;
+    }
+
+  _empty_queue(self, self->qoverflow);
+  _empty_queue(self, self->qout);
+  _empty_queue(self, self->qbacklog);
+
+  return result;
 }
 
 static gboolean
-_save_queue(LogQueueDisk *s, gboolean *persistent)
+_stop_corrupted(LogQueueDisk *s)
 {
-  gboolean success = FALSE;
-  LogQueueDiskNonReliable *self = (LogQueueDiskNonReliable *) s;
-  if (qdisk_save_state(s->qdisk, self->qout, self->qbacklog, self->qoverflow))
-    {
-      *persistent = TRUE;
-      success = TRUE;
-    }
-  qdisk_stop(s->qdisk);
-  return success;
-}
-
-static void
-_restart(LogQueueDisk *s, DiskQueueOptions *options)
-{
-  LogQueueDiskNonReliable *self = (LogQueueDiskNonReliable *) s;
-  qdisk_init_instance(self->super.qdisk, options, "SLQF");
+  return qdisk_stop(s->qdisk, NULL, NULL, NULL);
 }
 
 static inline void
@@ -539,9 +558,8 @@ static inline void
 _set_logqueue_disk_virtual_functions(LogQueueDisk *s)
 {
   s->start = _start;
-  s->load_queue = _load_queue;
-  s->save_queue = _save_queue;
-  s->restart = _restart;
+  s->stop = _stop;
+  s->stop_corrupted = _stop_corrupted;
 }
 
 static inline void
@@ -552,11 +570,14 @@ _set_virtual_functions(LogQueueDiskNonReliable *self)
 }
 
 LogQueue *
-log_queue_disk_non_reliable_new(DiskQueueOptions *options, const gchar *persist_name)
+log_queue_disk_non_reliable_new(DiskQueueOptions *options, const gchar *filename, const gchar *persist_name,
+                                gint stats_level, const StatsClusterKeyBuilder *driver_sck_builder,
+                                StatsClusterKeyBuilder *queue_sck_builder)
 {
   g_assert(options->reliable == FALSE);
   LogQueueDiskNonReliable *self = g_new0(LogQueueDiskNonReliable, 1);
-  log_queue_disk_init_instance(&self->super, options, "SLQF", persist_name);
+  log_queue_disk_init_instance(&self->super, options, "SLQF", filename, persist_name, stats_level,
+                               driver_sck_builder, queue_sck_builder);
   self->qbacklog = g_queue_new();
   self->qout = g_queue_new();
   self->qoverflow = g_queue_new();
