@@ -31,8 +31,12 @@ typedef struct _LogQueueDiskReliable
   LogQueueDisk super;
   GQueue *qreliable;
   GQueue *qbacklog;
+  GQueue *qout;
+  gint qout_size;
 } LogQueueDiskReliable;
 
-LogQueue *log_queue_disk_reliable_new(DiskQueueOptions *options);
+LogQueue *log_queue_disk_reliable_new(DiskQueueOptions *options, const gchar *filename, const gchar *persist_name,
+                                      gint stats_level, const StatsClusterKeyBuilder *driver_sck_builder,
+                                      StatsClusterKeyBuilder *queue_sck_builder);
 
 #endif /* LOGQUEUE_DISK_RELIABLE_H_ */
