@@ -35,7 +35,14 @@
  * uses this breaks, as it passes the result of a g_queue_pop_head call,
  * which has side effects.
  */
-#define POINTER_TO_LOG_PATH_OPTIONS(ptr, lpo) (lpo)->ack_needed = (GPOINTER_TO_INT(ptr) & ~0x80000000)
+#define POINTER_TO_LOG_PATH_OPTIONS(ptr, lpo) \
+({ \
+  gpointer p = ptr; \
+  (lpo)->ack_needed = (GPOINTER_TO_INT(p) & ~0x80000000); \
+})
+
+typedef gboolean (*QDiskSerializeFunc)(SerializeArchive *sa, gpointer user_data);
+typedef gboolean (*QDiskDeSerializeFunc)(SerializeArchive *sa, gpointer user_data);
 
 typedef struct
 {
@@ -47,40 +54,44 @@ QDiskQueuePosition;
 
 typedef struct _QDisk QDisk;
 
-QDisk *qdisk_new(void);
+QDisk *qdisk_new(DiskQueueOptions *options, const gchar *file_id, const gchar *filename);
 
 gboolean qdisk_is_space_avail(QDisk *self, gint at_least);
+gint64 qdisk_get_max_useful_space(QDisk *self);
 gint64 qdisk_get_empty_space(QDisk *self);
+gint64 qdisk_get_used_useful_space(QDisk *self);
 gboolean qdisk_push_tail(QDisk *self, GString *record);
 gboolean qdisk_pop_head(QDisk *self, GString *record);
-gboolean qdisk_start(QDisk *self, const gchar *filename, GQueue *qout, GQueue *qbacklog, GQueue *qoverflow);
-void qdisk_init_instance(QDisk *self, DiskQueueOptions *options, const gchar *file_id);
-void qdisk_stop(QDisk *self);
+gboolean qdisk_remove_head(QDisk *self);
+gboolean qdisk_ack_backlog(QDisk *self);
+gboolean qdisk_rewind_backlog(QDisk *self, guint rewind_count);
+void qdisk_empty_backlog(QDisk *self);
+gint64 qdisk_get_next_tail_position(QDisk *self);
+gint64 qdisk_get_next_head_position(QDisk *self);
+gboolean qdisk_start(QDisk *self, GQueue *qout, GQueue *qbacklog, GQueue *qoverflow);
+gboolean qdisk_stop(QDisk *self, GQueue *qout, GQueue *qbacklog, GQueue *qoverflow);
 void qdisk_reset_file_if_empty(QDisk *self);
 gboolean qdisk_started(QDisk *self);
 void qdisk_free(QDisk *self);
 
-gboolean qdisk_save_state(QDisk *self, GQueue *qout, GQueue *qbacklog, GQueue *qoverflow);
-
 DiskQueueOptions *qdisk_get_options(QDisk *self);
 gint64 qdisk_get_length(QDisk *self);
-void qdisk_set_length(QDisk *self, gint64 new_value);
 gint64 qdisk_get_maximum_size(QDisk *self);
 gint64 qdisk_get_writer_head(QDisk *self);
 gint64 qdisk_get_reader_head(QDisk *self);
-void qdisk_set_reader_head(QDisk *self, gint64 new_value);
 gint64 qdisk_get_backlog_head(QDisk *self);
-void qdisk_set_backlog_head(QDisk *self, gint64 new_value);
-void qdisk_inc_backlog(QDisk *self);
-void qdisk_dec_backlog(QDisk *self);
 gint64 qdisk_get_backlog_count(QDisk *self);
-void qdisk_set_backlog_count(QDisk *self, gint64 new_value);
 gint qdisk_get_memory_size(QDisk *self);
 gboolean qdisk_is_read_only(QDisk *self);
 const gchar *qdisk_get_filename(QDisk *self);
+gint64 qdisk_get_file_size(QDisk *self);
 
-gssize qdisk_read(QDisk *self, gpointer buffer, gsize bytes_to_read, gint64 position);
-guint64 qdisk_skip_record(QDisk *self, guint64 position);
-ssize_t pread_all(int fd, char *buf,  size_t count, off_t offset, gint max_block);
+gchar *qdisk_get_next_filename(const gchar *dir, gboolean reliable);
+gboolean qdisk_is_file_a_disk_buffer_file(const gchar *filename);
+gboolean qdisk_is_disk_buffer_file_reliable(const gchar *filename, gboolean *reliable);
+
+gboolean qdisk_serialize(GString *serialized, QDiskSerializeFunc serialize_func, gpointer user_data, GError **error);
+gboolean qdisk_deserialize(GString *serialized, QDiskDeSerializeFunc deserialize_func, gpointer user_data,
+                           GError **error);
 
 #endif /* QDISK_H_ */
