@@ -62,13 +62,13 @@ Test(diskq, testcase_zero_diskbuf_and_normal_acks)
 
   _construct_options(&options, 10000000, 100000, TRUE);
 
-  q = log_queue_disk_reliable_new(&options, NULL);
-  log_queue_set_use_backlog(q, TRUE);
-
   filename = g_string_sized_new(32);
   g_string_printf(filename, "test-normal_acks.qf");
   unlink(filename->str);
-  log_queue_disk_load_queue(q, filename->str);
+  q = log_queue_disk_reliable_new(&options, filename->str, NULL, STATS_LEVEL0, NULL, NULL);
+  log_queue_set_use_backlog(q, TRUE);
+
+  log_queue_disk_start(q);
   fed_messages = 0;
   acked_messages = 0;
   for (i = 0; i < 10; i++)
@@ -80,6 +80,8 @@ Test(diskq, testcase_zero_diskbuf_and_normal_acks)
                "%s: did not receive enough acknowledgements: fed_messages=%d, acked_messages=%d\n", __FUNCTION__, fed_messages,
                acked_messages);
 
+  gboolean persistent;
+  log_queue_disk_stop(q, &persistent);
   log_queue_unref(q);
   unlink(filename->str);
   g_string_free(filename, TRUE);
@@ -95,13 +97,13 @@ Test(diskq, testcase_zero_diskbuf_alternating_send_acks)
 
   _construct_options(&options, 10000000, 100000, TRUE);
 
-  q = log_queue_disk_reliable_new(&options, NULL);
-  log_queue_set_use_backlog(q, TRUE);
-
   filename = g_string_sized_new(32);
   g_string_printf(filename, "test-send_acks.qf");
   unlink(filename->str);
-  log_queue_disk_load_queue(q, filename->str);
+  q = log_queue_disk_reliable_new(&options, filename->str, NULL, STATS_LEVEL0, NULL, NULL);
+  log_queue_set_use_backlog(q, TRUE);
+  log_queue_disk_start(q);
+
   fed_messages = 0;
   acked_messages = 0;
   for (i = 0; i < 10; i++)
@@ -114,6 +116,9 @@ Test(diskq, testcase_zero_diskbuf_alternating_send_acks)
   cr_assert_eq(fed_messages, acked_messages,
                "%s: did not receive enough acknowledgements: fed_messages=%d, acked_messages=%d\n", __FUNCTION__, fed_messages,
                acked_messages);
+
+  gboolean persistent;
+  log_queue_disk_stop(q, &persistent);
   log_queue_unref(q);
   unlink(filename->str);
   g_string_free(filename, TRUE);
@@ -129,42 +134,43 @@ Test(diskq, testcase_ack_and_rewind_messages)
 
   _construct_options(&options, 10000000, 100000, TRUE);
 
-  q = log_queue_disk_reliable_new(&options, NULL);
-  log_queue_set_use_backlog(q, TRUE);
-
-  StatsClusterKey sc_key;
-  stats_cluster_logpipe_key_legacy_set(&sc_key, SCS_DESTINATION, "queued messages", NULL);
-  stats_lock();
-  stats_register_counter(0, &sc_key, SC_TYPE_QUEUED, &q->queued_messages);
-  stats_unlock();
-
-  cr_assert_eq(stats_counter_get(q->queued_messages), 0, "queued messages: %d", __LINE__);
-
   filename = g_string_sized_new(32);
   g_string_printf(filename, "test-rewind_and_acks.qf");
   unlink(filename->str);
-  log_queue_disk_load_queue(q, filename->str);
+  StatsClusterKeyBuilder *driver_sck_builder = stats_cluster_key_builder_new();
+  StatsClusterKeyBuilder *queue_sck_builder = stats_cluster_key_builder_new();
+  q = log_queue_disk_reliable_new(&options, filename->str, NULL, STATS_LEVEL0, driver_sck_builder, queue_sck_builder);
+  stats_cluster_key_builder_free(driver_sck_builder);
+  stats_cluster_key_builder_free(queue_sck_builder);
+  log_queue_set_use_backlog(q, TRUE);
+
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 0, "queued messages: %d", __LINE__);
+
+  log_queue_disk_start(q);
 
   fed_messages = 0;
   acked_messages = 0;
   feed_some_messages(q, 1000);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 1000, "queued messages: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 1000, "queued messages: %d", __LINE__);
 
   for (i = 0; i < 10; i++)
     {
       send_some_messages(q, 1);
-      cr_assert_eq(stats_counter_get(q->queued_messages), 999, "queued messages wrong number %d", __LINE__);
+      cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 999, "queued messages wrong number %d", __LINE__);
       log_queue_rewind_backlog(q, 1);
-      cr_assert_eq(stats_counter_get(q->queued_messages), 1000, "queued messages wrong number: %d", __LINE__);
+      cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 1000, "queued messages wrong number: %d", __LINE__);
     }
   send_some_messages(q, 1000);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 0, "queued messages: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 0, "queued messages: %d", __LINE__);
   log_queue_ack_backlog(q, 500);
   log_queue_rewind_backlog(q, 500);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 500, "queued messages: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 500, "queued messages: %d", __LINE__);
   send_some_messages(q, 500);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 0, "queued messages: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 0, "queued messages: %d", __LINE__);
   log_queue_ack_backlog(q, 500);
+
+  gboolean persistent;
+  log_queue_disk_stop(q, &persistent);
   log_queue_unref(q);
   unlink(filename->str);
   g_string_free(filename, TRUE);
@@ -279,11 +285,11 @@ Test(diskq, testcase_with_threads)
 
       _construct_options(&options, 10000000, 100000, TRUE);
 
-      q = log_queue_disk_reliable_new(&options, NULL);
       filename = g_string_sized_new(32);
       g_string_printf(filename, "test-%04d.qf", i);
       unlink(filename->str);
-      log_queue_disk_load_queue(q, filename->str);
+      q = log_queue_disk_reliable_new(&options, filename->str, NULL, STATS_LEVEL0, NULL, NULL);
+      log_queue_disk_start(q);
 
       for (j = 0; j < FEEDERS; j++)
         {
@@ -298,6 +304,8 @@ Test(diskq, testcase_with_threads)
         }
       g_thread_join(thread_consume);
 
+      gboolean persistent;
+      log_queue_disk_stop(q, &persistent);
       log_queue_unref(q);
       unlink(filename->str);
       g_string_free(filename, TRUE);
@@ -314,12 +322,12 @@ typedef struct restart_test_parameters
 } restart_test_parameters;
 
 static LogQueue *
-queue_new(gboolean reliable, DiskQueueOptions *options, const gchar *persist_name)
+queue_new(gboolean reliable, DiskQueueOptions *options, const gchar *filename, const gchar *persist_name)
 {
   if (reliable)
-    return log_queue_disk_reliable_new(options, persist_name);
+    return log_queue_disk_reliable_new(options, filename, persist_name, STATS_LEVEL0, NULL, NULL);
 
-  return log_queue_disk_non_reliable_new(options, persist_name);
+  return log_queue_disk_non_reliable_new(options, filename, persist_name, STATS_LEVEL0, NULL, NULL);
 }
 
 ParameterizedTestParameters(diskq, testcase_diskbuffer_restart_corrupted)
@@ -338,16 +346,16 @@ ParameterizedTest(restart_test_parameters *test_case, diskq, testcase_diskbuffer
   guint64 const original_disk_buf_size = 1000123;
   DiskQueueOptions options;
   _construct_options(&options, original_disk_buf_size, 100000, test_case->reliable);
-  LogQueue *q = queue_new(test_case->reliable, &options, NULL);
+  gchar *filename = test_case->filename;
+  LogQueue *q = queue_new(test_case->reliable, &options, filename, NULL);
   log_queue_set_use_backlog(q, FALSE);
 
-  gchar *filename = test_case->filename;
   gchar filename_corrupted_dq[100];
   g_snprintf(filename_corrupted_dq, 100, "%s.corrupted", filename);
   unlink(filename);
   unlink(filename_corrupted_dq);
 
-  log_queue_disk_load_queue(q, filename);
+  log_queue_disk_start(q);
   fed_messages = 0;
   feed_some_messages(q, 100);
   cr_assert_eq(fed_messages, 100, "Failed to push all messages to the disk-queue!\n");
@@ -375,6 +383,8 @@ ParameterizedTest(restart_test_parameters *test_case, diskq, testcase_diskbuffer
   cr_assert_eq(qdisk_get_reader_head(disk_queue->qdisk), QDISK_RESERVED_SPACE,
                "Invalid read pointer!\n");
 
+  gboolean persistent;
+  log_queue_disk_stop(q, &persistent);
   log_queue_unref(q);
   unlink(filename);
   unlink(filename_corrupted_dq);
@@ -405,39 +415,24 @@ typedef struct diskq_tester_parameters
   const gchar *filename;
 } diskq_tester_parameters_t;
 
-void
-init_statistics(LogQueue *q)
-{
-
-  StatsClusterKey sc_key1, sc_key2;
-  stats_lock();
-  stats_cluster_logpipe_key_legacy_set(&sc_key1, SCS_DESTINATION, "queued messages", NULL);
-  stats_register_counter(0, &sc_key1, SC_TYPE_QUEUED, &q->queued_messages);
-  stats_cluster_single_key_legacy_set_with_name(&sc_key2, SCS_DESTINATION, "memory usage", NULL, "memory_usage");
-  stats_register_counter(1, &sc_key2, SC_TYPE_SINGLE_VALUE, &q->memory_usage);
-  stats_unlock();
-  stats_counter_set(q->queued_messages, 0);
-  stats_counter_set(q->memory_usage, 0);
-}
-
 static void
 assert_general_message_flow(LogQueue *q, gssize one_msg_size)
 {
   send_some_messages(q, 1);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 1, "queued messages: line: %d", __LINE__);
-  cr_assert_eq(stats_counter_get(q->memory_usage), one_msg_size, "memory_usage: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 1, "queued messages: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.memory_usage), one_msg_size, "memory_usage: line: %d", __LINE__);
 
   send_some_messages(q, 1);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 0, "queued messages: line: %d", __LINE__);
-  cr_assert_eq(stats_counter_get(q->memory_usage), 0, "memory_usage: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 0, "queued messages: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.memory_usage), 0, "memory_usage: line: %d", __LINE__);
 
   feed_some_messages(q, 10);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 10, "queued messages: line: %d", __LINE__);
-  cr_assert_eq(stats_counter_get(q->memory_usage), one_msg_size*10, "memory_usage: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 10, "queued messages: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.memory_usage), one_msg_size*10, "memory_usage: line: %d", __LINE__);
 
   send_some_messages(q, 5);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 5, "queued messages: line: %d", __LINE__);
-  cr_assert_eq(stats_counter_get(q->memory_usage), one_msg_size*5, "memory_usage: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 5, "queued messages: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.memory_usage), one_msg_size*5, "memory_usage: line: %d", __LINE__);
 }
 
 static LogQueue *
@@ -448,17 +443,22 @@ testcase_diskq_prepare(DiskQueueOptions *options, diskq_tester_parameters_t *par
   _construct_options(options, parameters->disk_size, 100000, parameters->reliable);
   options->qout_size = parameters->qout_size;
 
+  StatsClusterKeyBuilder *driver_sck_builder = stats_cluster_key_builder_new();
+  StatsClusterKeyBuilder *queue_sck_builder = stats_cluster_key_builder_new();
   if (parameters->reliable)
-    q = log_queue_disk_reliable_new(options, NULL);
+    q = log_queue_disk_reliable_new(options, parameters->filename, NULL, STATS_LEVEL0, driver_sck_builder,
+                                    queue_sck_builder);
   else
-    q = log_queue_disk_non_reliable_new(options, NULL);
+    q = log_queue_disk_non_reliable_new(options, parameters->filename, NULL, STATS_LEVEL0, driver_sck_builder,
+                                        queue_sck_builder);
+  stats_cluster_key_builder_free(driver_sck_builder);
+  stats_cluster_key_builder_free(queue_sck_builder);
 
-  init_statistics(q);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 0, "queued messages: line: %d", __LINE__);
-  cr_assert_eq(stats_counter_get(q->memory_usage), 0, "memory_usage: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 0, "queued messages: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.memory_usage), 0, "memory_usage: line: %d", __LINE__);
 
   unlink(parameters->filename);
-  log_queue_disk_load_queue(q, parameters->filename);
+  log_queue_disk_start(q);
 
   return q;
 }
@@ -504,22 +504,22 @@ ParameterizedTest(diskq_tester_parameters_t *parameters, diskq, test_diskq_stati
   q = testcase_diskq_prepare(&options, parameters);
 
   feed_some_messages(q, 1);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 1, "queued messages: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 1, "queued messages: line: %d", __LINE__);
 
   if (parameters->overflow_expected)
     assert_overflow_queue_length(parameters, q, 1);
 
-  guint32 one_msg_size = stats_counter_get(q->memory_usage);
+  guint32 one_msg_size = stats_counter_get(q->metrics.shared.memory_usage);
   if (parameters->overflow_expected)
     /* Only when overflow. If there is no overflow, the first
        msg is put to the output queue so statistics is not increased: one_msg_size == 0 */
     cr_assert(is_valid_msg_size(one_msg_size), "one_msg_size %d: line: %d", one_msg_size, __LINE__);
   else
-    cr_assert_eq(stats_counter_get(q->memory_usage), 0, "queued messages: line: %d", __LINE__);
+    cr_assert_eq(stats_counter_get(q->metrics.shared.memory_usage), 0, "queued messages: line: %d", __LINE__);
 
   feed_some_messages(q, 1);
-  cr_assert_eq(stats_counter_get(q->queued_messages), 2, "queued messages: line: %d", __LINE__);
-  cr_assert_eq(stats_counter_get(q->memory_usage), one_msg_size*2, "memory_usage: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.queued_messages), 2, "queued messages: line: %d", __LINE__);
+  cr_assert_eq(stats_counter_get(q->metrics.shared.memory_usage), one_msg_size*2, "memory_usage: line: %d", __LINE__);
 
   if (parameters->overflow_expected)
     assert_overflow_queue_length(parameters, q, 2);
@@ -528,10 +528,41 @@ ParameterizedTest(diskq_tester_parameters_t *parameters, diskq, test_diskq_stati
 
   unlink(parameters->filename);
 
+  gboolean persistent;
+  log_queue_disk_stop(q, &persistent);
   log_queue_unref(q);
   disk_queue_options_destroy(&options);
 }
 
+gchar *
+qdisk_get_next_filename(const gchar *dir, gboolean reliable)
+{
+  return NULL;
+}
+
+Test(diskq, test_no_next_filename_in_acquire)
+{
+  const gchar *queue_persist_name = "test_no_next_filename_in_acquire";
+  const gchar *persist_filename = "test_no_next_filename_in_acquire.persist";
+  configuration->state = persist_state_new(persist_filename);
+  persist_state_start(configuration->state);
+
+  LogDestDriver *driver = g_new0(LogDestDriver, 1);
+  log_dest_driver_init_instance(driver, configuration);
+
+  DiskQDestPlugin *plugin = diskq_dest_plugin_new();
+  cr_assert(log_driver_add_plugin(&driver->super, (LogDriverPlugin *) plugin));
+  cr_assert(log_pipe_init(&driver->super.super));
+
+  cr_assert_eq(log_dest_driver_acquire_queue(driver, queue_persist_name, STATS_LEVEL0, NULL, NULL), NULL);
+
+  cr_assert(log_pipe_deinit(&driver->super.super));
+  cr_assert(log_pipe_unref(&driver->super.super));
+
+  persist_state_cancel(configuration->state);
+  unlink(persist_filename);
+}
+
 static void
 setup(void)
 {
