@@ -43,9 +43,9 @@
 static LogQueue *
 _get_non_reliable_diskqueue(gchar *filename, DiskQueueOptions *options)
 {
-  LogQueue *q = log_queue_disk_non_reliable_new(options, NULL);
+  LogQueue *q = log_queue_disk_non_reliable_new(options, filename, NULL, STATS_LEVEL0, NULL, NULL);
   log_queue_set_use_backlog(q, FALSE);
-  log_queue_disk_load_queue(q, filename);
+  log_queue_disk_start(q);
   return q;
 }
 
@@ -71,7 +71,7 @@ static void
 _save_diskqueue(LogQueue *q)
 {
   gboolean persistent;
-  log_queue_disk_save_queue(q, &persistent);
+  log_queue_disk_stop(q, &persistent);
   log_queue_unref(q);
 }
 
@@ -138,9 +138,9 @@ _assert_diskq_actual_file_size_with_stored(LogQueue *q)
   QDisk *qdisk = ((LogQueueDisk *)q)->qdisk;
 
   gint64 actual_file_size = _get_file_size(q);
-  cr_assert_eq(qdisk->file_size, actual_file_size,
+  cr_assert_eq(qdisk->cached_file_size, actual_file_size,
                "File size does not match with stored size; Actual file size: %ld, Expected file size: %ld\n", actual_file_size,
-               qdisk->file_size);
+               qdisk->cached_file_size);
 }
 
 static void
@@ -200,7 +200,7 @@ _test_diskq_truncate(TruncateTestParams params)
 
   unlink(params.filename);
 
-  log_queue_unref(q);
+  _save_diskqueue(q);
   disk_queue_options_destroy(&options);
 }
 
@@ -242,9 +242,9 @@ _create_reliable_diskqueue(gchar *filename, DiskQueueOptions *options, gboolean
     truncate_size_ratio = disk_queue_config_get_truncate_size_ratio(configuration);
   options->truncate_size_ratio = truncate_size_ratio;
 
-  q = log_queue_disk_reliable_new(options, "persist-name");
+  q = log_queue_disk_reliable_new(options, filename, "persist-name", STATS_LEVEL0, NULL, NULL);
   log_queue_set_use_backlog(q, use_backlog);
-  log_queue_disk_load_queue(q, filename);
+  log_queue_disk_start(q);
   return q;
 }
 
@@ -506,7 +506,7 @@ Test(diskq_truncate, test_diskq_no_truncate_wrap)
   _feed_one_large_message(q);
   QDisk *qdisk = ((LogQueueDisk *)q)->qdisk;
   cr_assert(qdisk_get_writer_head(qdisk) < qdisk_get_reader_head(qdisk), "write_head should have wrapped");
-  cr_assert(qdisk->file_size > TEST_DISKQ_SIZE, "file_size should be bigger than max size");
+  cr_assert(qdisk->cached_file_size > TEST_DISKQ_SIZE, "file_size should be bigger than max size");
   unprocessed_messages_in_buffer += 1;
 
   // 4. send and ack all messages
