@@ -21,9 +21,10 @@
  *
  */
 
-#include "queue_utils_lib.h"
+#include <criterion/criterion.h>
+#include "libtest/queue_utils_lib.h"
 #include "test_diskq_tools.h"
-#include "testutils.h"
+
 #include "qdisk.c"
 #include "logqueue-disk-reliable.h"
 #include "apphook.h"
@@ -34,9 +35,7 @@
 #include <unistd.h>
 
 
-MsgFormatOptions parse_options;
 
-#define FILENAME "test_reliable_backlog.qf"
 #define TEST_DISKQ_SIZE QDISK_RESERVED_SPACE + 1000 /* 4096 + 1000 */
 
 static gint num_of_ack;
@@ -47,8 +46,6 @@ DiskQueueOptions options;
 
 #define NUMBER_MESSAGES_IN_QUEUE(n) (n * 3)
 
-MsgFormatOptions parse_options;
-
 static void
 _dummy_ack(LogMessage *lm,  AckType ack_type)
 {
@@ -56,32 +53,33 @@ _dummy_ack(LogMessage *lm,  AckType ack_type)
 }
 
 static LogQueueDiskReliable *
-_init_diskq_for_test(gint64 size, gint64 membuf_size)
+_init_diskq_for_test(const gchar *filename, gint64 size, gint64 membuf_size)
 {
   LogQueueDiskReliable *dq;
 
   _construct_options(&options, size, membuf_size, TRUE);
-  LogQueue *q = log_queue_disk_reliable_new(&options);
+  LogQueue *q = log_queue_disk_reliable_new(&options, filename, NULL, STATS_LEVEL0, NULL, NULL);
   struct stat st;
   num_of_ack = 0;
-  gchar *filename = FILENAME;
   unlink(filename);
-  log_queue_disk_load_queue(q, filename);
+  log_queue_disk_start(q);
   dq = (LogQueueDiskReliable *)q;
   lseek(dq->super.qdisk->fd, size - 1, SEEK_SET);
   ssize_t written = write(dq->super.qdisk->fd, "", 1);
-  assert_gint(written, 1, ASSERTION_ERROR("Can't write to diskq file"));
+  cr_assert_eq(written, 1, "%s", "Can't write to diskq file");
   fstat(dq->super.qdisk->fd, &st);
-  assert_gint64(st.st_size, size, ASSERTION_ERROR("INITIALIZATION FAILED"));
+  cr_assert_eq(st.st_size, size, "%s", "INITIALIZATION FAILED");
   dq->super.super.use_backlog = TRUE;
   return dq;
 }
 
 static void
-_common_cleanup(LogQueueDiskReliable *dq)
+_common_cleanup(LogQueueDiskReliable *dq, const gchar *file_name)
 {
+  gboolean persistent;
+  log_queue_disk_stop(&dq->super.super, &persistent);
   log_queue_unref(&dq->super.super);
-  unlink(FILENAME);
+  unlink(file_name);
   disk_queue_options_destroy(&options);
 }
 
@@ -95,7 +93,7 @@ get_serialized_message_size(LogMessage *msg)
   serialized = g_string_sized_new(64);
   sa = serialize_string_archive_new(serialized);
 
-  assert_true(log_msg_serialize(msg, sa), NULL);
+  cr_assert(log_msg_serialize(msg, sa, 0));
 
   result = serialized->len;
 
@@ -106,7 +104,7 @@ get_serialized_message_size(LogMessage *msg)
 }
 
 static void
-set_mark_message_serialized_size()
+set_mark_message_serialized_size(void)
 {
   LogMessage *mark_message = log_msg_new_mark();
   mark_message_serialized_size = get_serialized_message_size(mark_message);
@@ -117,7 +115,7 @@ static void
 _prepare_eof_test(LogQueueDiskReliable *dq, LogMessage **msg1, LogMessage **msg2)
 {
   LogPathOptions local_options = LOG_PATH_OPTIONS_INIT;
-  gint64 start_pos = TEST_DISKQ_SIZE;
+  gint64 start_pos = TEST_DISKQ_SIZE - 1;
 
   *msg1 = log_msg_new_mark();
   *msg2 = log_msg_new_mark();
@@ -135,10 +133,10 @@ _prepare_eof_test(LogQueueDiskReliable *dq, LogMessage **msg1, LogMessage **msg2
   log_queue_push_tail(&dq->super.super, *msg1, &local_options);
   log_queue_push_tail(&dq->super.super, *msg2, &local_options);
 
-  assert_gint(dq->qreliable->length, NUMBER_MESSAGES_IN_QUEUE(2), ASSERTION_ERROR("Messages aren't in qreliable"));
-  assert_gint64(dq->super.qdisk->hdr->write_head, QDISK_RESERVED_SPACE + mark_message_serialized_size,
-                ASSERTION_ERROR("Bad write head"));
-  assert_gint(num_of_ack, 0, ASSERTION_ERROR("Messages are acked"));
+  cr_assert_eq(dq->qreliable->length, NUMBER_MESSAGES_IN_QUEUE(2), "%s", "Messages aren't in qreliable");
+  cr_assert_eq(dq->super.qdisk->hdr->write_head, QDISK_RESERVED_SPACE + mark_message_serialized_size,
+               "%s", "Bad write head");
+  cr_assert_eq(num_of_ack, 0, "%s", "Messages are acked");
 
   dq->super.qdisk->hdr->read_head = start_pos;
   dq->super.qdisk->hdr->backlog_head = dq->super.qdisk->hdr->read_head;
@@ -153,15 +151,15 @@ test_read_over_eof(LogQueueDiskReliable *dq, LogMessage *msg1, LogMessage *msg2)
   LogMessage *read_message2;
 
   read_message1 = log_queue_pop_head(&dq->super.super, &read_options);
-  assert_true(read_message1 != NULL, ASSERTION_ERROR("Can't read message from queue"));
+  cr_assert_not_null(read_message1, "%s", "Can't read message from queue");
   read_message2 = log_queue_pop_head(&dq->super.super, &read_options);
-  assert_true(read_message2 != NULL, ASSERTION_ERROR("Can't read message from queue"));
-  assert_gint(dq->qreliable->length, 0, ASSERTION_ERROR("Queue reliable isn't empty"));
-  assert_gint(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(2), ASSERTION_ERROR("Messages aren't in the qbacklog"));
-  assert_gint(dq->super.qdisk->hdr->read_head, dq->super.qdisk->hdr->write_head,
-              ASSERTION_ERROR("Read head in bad position"));
-  assert_true(msg1 == read_message1, ASSERTION_ERROR("Message 1 isn't read from qreliable"));
-  assert_true(msg2 == read_message2, ASSERTION_ERROR("Message 2 isn't read from qreliable"));
+  cr_assert_not_null(read_message2, "%s", "Can't read message from queue");
+  cr_assert_eq(dq->qreliable->length, 0, "%s", "Queue reliable isn't empty");
+  cr_assert_eq(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(2), "%s", "Messages aren't in the qbacklog");
+  cr_assert_eq(dq->super.qdisk->hdr->read_head, dq->super.qdisk->hdr->write_head,
+               "%s", "Read head in bad position");
+  cr_assert_eq(msg1, read_message1, "%s", "Message 1 isn't read from qreliable");
+  cr_assert_eq(msg2, read_message2, "%s", "Message 2 isn't read from qreliable");
 }
 
 static void
@@ -176,22 +174,22 @@ test_rewind_over_eof(LogQueueDiskReliable *dq)
   log_queue_push_tail(&dq->super.super, msg3, &local_options);
   gint64 previous_read_head = dq->super.qdisk->hdr->read_head;
   read_message3 = log_queue_pop_head(&dq->super.super, &local_options);
-  assert_true(read_message3 != NULL, ASSERTION_ERROR("Can't read message from queue"));
-  assert_gint(dq->super.qdisk->hdr->read_head, dq->super.qdisk->hdr->write_head,
-              ASSERTION_ERROR("Read head in bad position"));
+  cr_assert_not_null(read_message3, "%s", "Can't read message from queue");
+  cr_assert_eq(dq->super.qdisk->hdr->read_head, dq->super.qdisk->hdr->write_head,
+               "%s", "Read head in bad position");
 
-  assert_true(msg3 == read_message3, ASSERTION_ERROR("Message 3 isn't read from qreliable"));
+  cr_assert_eq(msg3, read_message3, "%s", "Message 3 isn't read from qreliable");
   log_msg_unref(read_message3);
 
   log_queue_rewind_backlog(&dq->super.super, 1);
 
-  assert_gint(dq->super.qdisk->hdr->read_head, previous_read_head, ASSERTION_ERROR("Read head is corrupted"));
+  cr_assert_eq(dq->super.qdisk->hdr->read_head, previous_read_head, "%s", "Read head is corrupted");
 
   read_message3 = log_queue_pop_head(&dq->super.super, &local_options);
-  assert_true(read_message3 != NULL, ASSERTION_ERROR("Can't read message from queue"));
-  assert_gint(dq->super.qdisk->hdr->read_head, dq->super.qdisk->hdr->write_head,
-              ASSERTION_ERROR("Read head in bad position"));
-  assert_true(msg3 == read_message3, ASSERTION_ERROR("Message 3 isn't read from qreliable"));
+  cr_assert_not_null(read_message3, "%s", "Can't read message from queue");
+  cr_assert_eq(dq->super.qdisk->hdr->read_head, dq->super.qdisk->hdr->write_head,
+               "%s", "Read head in bad position");
+  cr_assert_eq(msg3, read_message3, "%s", "Message 3 isn't read from qreliable");
 
   log_msg_drop(msg3, &local_options, AT_PROCESSED);
 }
@@ -200,9 +198,9 @@ static void
 test_ack_over_eof(LogQueueDiskReliable *dq, LogMessage *msg1, LogMessage *msg2)
 {
   log_queue_ack_backlog(&dq->super.super, 3);
-  assert_gint(dq->qbacklog->length, 0, ASSERTION_ERROR("Messages are in the qbacklog"));
-  assert_gint(dq->super.qdisk->hdr->backlog_head, dq->super.qdisk->hdr->read_head,
-              ASSERTION_ERROR("Backlog head in bad position"));
+  cr_assert_eq(dq->qbacklog->length, 0, "%s", "Messages are in the qbacklog");
+  cr_assert_eq(dq->super.qdisk->hdr->backlog_head, dq->super.qdisk->hdr->read_head,
+               "%s", "Backlog head in bad position");
 }
 
 /* TestCase:
@@ -215,10 +213,11 @@ test_ack_over_eof(LogQueueDiskReliable *dq, LogMessage *msg1, LogMessage *msg2)
    */
 /* TODO: add 3 messages and rewind 1 instead of 0 */
 /* TODO: split this test into 3 tests (read ack rewind mechanism  (setup method) */
-static void
-test_over_EOF()
+Test(diskq_reliable, test_over_EOF)
 {
-  LogQueueDiskReliable *dq = _init_diskq_for_test(TEST_DISKQ_SIZE, TEST_DISKQ_SIZE);
+  const gchar *file_name = "test_over_EOF.rqf";
+
+  LogQueueDiskReliable *dq = _init_diskq_for_test(file_name, TEST_DISKQ_SIZE, TEST_DISKQ_SIZE);
   LogMessage *msg1;
   LogMessage *msg2;
 
@@ -234,8 +233,8 @@ test_over_EOF()
 
   log_msg_drop(msg1, &read_options, AT_PROCESSED);
   log_msg_drop(msg2, &read_options, AT_PROCESSED);
-  assert_gint(num_of_ack, 2, ASSERTION_ERROR("Messages aren't acked"));
-  _common_cleanup(dq);
+  cr_assert_eq(num_of_ack, 2, "%s", "Messages aren't acked");
+  _common_cleanup(dq, file_name);
 }
 
 /*
@@ -269,8 +268,8 @@ _prepare_rewind_backlog_test(LogQueueDiskReliable *dq, gint64 *start_pos)
 
   /* Ack the messages which are not in the qbacklog */
   log_queue_ack_backlog(&dq->super.super, 5);
-  assert_gint(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(3),
-              ASSERTION_ERROR("Incorrect number of items in the qbacklog"));
+  cr_assert_eq(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(3),
+               "%s", "Incorrect number of items in the qbacklog");
 
   *start_pos = dq->super.qdisk->hdr->read_head;
 
@@ -288,16 +287,16 @@ _prepare_rewind_backlog_test(LogQueueDiskReliable *dq, gint64 *start_pos)
       mark_message->ack_func = _dummy_ack;
       log_queue_push_tail(&dq->super.super, mark_message, &path_options);
       mark_message = log_queue_pop_head(&dq->super.super, &path_options);
-      assert_gint(dq->qreliable->length, 0,
-                  ASSERTION_ERROR("Incorrect number of items in the qreliable"));
-      assert_gint(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(3),
-                  ASSERTION_ERROR("Incorrect number of items in the qbacklog"));
+      cr_assert_eq(dq->qreliable->length, 0,
+                   "%s", "Incorrect number of items in the qreliable");
+      cr_assert_eq(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(3),
+                   "%s", "Incorrect number of items in the qbacklog");
       log_msg_unref(mark_message);
     }
-  assert_gint(dq->super.qdisk->hdr->backlog_len, 6,
-              ASSERTION_ERROR("Incorrect number of messages in the backlog"));
-  assert_gint(dq->super.qdisk->hdr->length, 0,
-              ASSERTION_ERROR("Reliable diskq isn't empty"));
+  cr_assert_eq(dq->super.qdisk->hdr->backlog_len, 6,
+               "%s", "Incorrect number of messages in the backlog");
+  cr_assert_eq(dq->super.qdisk->hdr->length, 0,
+               "%s", "Reliable diskq isn't empty");
 }
 
 void
@@ -309,11 +308,11 @@ test_rewind_backlog_without_using_qbacklog(LogQueueDiskReliable *dq, gint64 old_
      * - the qbacklog and qreliable should be untouched
      */
   log_queue_rewind_backlog(&dq->super.super, 2);
-  assert_gint64(dq->super.qdisk->hdr->read_head, old_read_pos + mark_message_serialized_size,
-                ASSERTION_ERROR("Bad reader position"));
-  assert_gint(dq->qreliable->length, 0, ASSERTION_ERROR("Incorrect number of items in the qreliable"));
-  assert_gint(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(3),
-              ASSERTION_ERROR("Incorrect number of items in the qbacklog"));
+  cr_assert_eq(dq->super.qdisk->hdr->read_head, old_read_pos + mark_message_serialized_size,
+               "%s", "Bad reader position");
+  cr_assert_eq(dq->qreliable->length, 0, "%s", "Incorrect number of items in the qreliable");
+  cr_assert_eq(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(3),
+               "%s", "Incorrect number of items in the qbacklog");
 }
 
 void
@@ -326,12 +325,12 @@ test_rewind_backlog_partially_used_qbacklog(LogQueueDiskReliable *dq, gint64 old
    * - the qbackbacklog should contain 2 items
    */
   log_queue_rewind_backlog(&dq->super.super, 2);
-  assert_gint64(dq->super.qdisk->hdr->read_head, old_read_pos - mark_message_serialized_size,
-                ASSERTION_ERROR("Bad reader position"));
-  assert_gint(dq->qreliable->length, NUMBER_MESSAGES_IN_QUEUE(1),
-              ASSERTION_ERROR("Incorrect number of items in the qreliable"));
-  assert_gint(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(2),
-              ASSERTION_ERROR("Incorrect number of items in the qbacklog"));
+  cr_assert_eq(dq->super.qdisk->hdr->read_head, old_read_pos - mark_message_serialized_size,
+               "%s", "Bad reader position");
+  cr_assert_eq(dq->qreliable->length, NUMBER_MESSAGES_IN_QUEUE(1),
+               "%s", "Incorrect number of items in the qreliable");
+  cr_assert_eq(dq->qbacklog->length, NUMBER_MESSAGES_IN_QUEUE(2),
+               "%s", "Incorrect number of items in the qbacklog");
 }
 
 void
@@ -344,12 +343,12 @@ test_rewind_backlog_use_whole_qbacklog(LogQueueDiskReliable *dq)
    * - the qbackbacklog should be empty
    */
   log_queue_rewind_backlog(&dq->super.super, 2);
-  assert_gint64(dq->super.qdisk->hdr->read_head, dq->super.qdisk->hdr->backlog_head,
-                ASSERTION_ERROR("Bad reader position"));
-  assert_gint(dq->qreliable->length, NUMBER_MESSAGES_IN_QUEUE(3),
-              ASSERTION_ERROR("Incorrect number of items in the qreliable"));
-  assert_gint(dq->qbacklog->length, 0,
-              ASSERTION_ERROR("Incorrect number of items in the qbacklog"));
+  cr_assert_eq(dq->super.qdisk->hdr->read_head, dq->super.qdisk->hdr->backlog_head,
+               "%s", "Bad reader position");
+  cr_assert_eq(dq->qreliable->length, NUMBER_MESSAGES_IN_QUEUE(3),
+               "%s", "Incorrect number of items in the qreliable");
+  cr_assert_eq(dq->qbacklog->length, 0,
+               "%s", "Incorrect number of items in the qbacklog");
 
 }
 
@@ -359,11 +358,11 @@ test_rewind_backlog_use_whole_qbacklog(LogQueueDiskReliable *dq)
  * qbacklog contains messages: 1 2 3
  * qbacklog must be always in sync with backlog
  */
-void
-test_rewind_backlog()
+Test(diskq_reliable, test_rewind_backlog)
 {
-  LogQueueDiskReliable *dq = _init_diskq_for_test(QDISK_RESERVED_SPACE + mark_message_serialized_size * 10,
-                             mark_message_serialized_size * 5);
+  const gchar *file_name = "test_rewind_backlog.rqf";
+  LogQueueDiskReliable *dq = _init_diskq_for_test(file_name, QDISK_RESERVED_SPACE + mark_message_serialized_size * 10 + 1,
+                                                  mark_message_serialized_size * 5);
   gint64 old_read_pos;
 
   _prepare_rewind_backlog_test(dq, &old_read_pos);
@@ -374,29 +373,27 @@ test_rewind_backlog()
 
   test_rewind_backlog_use_whole_qbacklog(dq);
 
-  _common_cleanup(dq);
+  _common_cleanup(dq, file_name);
 }
 
-gint
-main(gint argc, gchar **argv)
+static void
+setup(void)
 {
   app_startup();
-  putenv("TZ=MET-1METDST");
+  setenv("TZ", "MET-1METDST", TRUE);
   tzset();
 
-  configuration = cfg_new(0x0308);
-  plugin_load_module("syslogformat", configuration, NULL);
-  plugin_load_module("disk-buffer", configuration, NULL);
-  msg_format_options_defaults(&parse_options);
-  msg_format_options_init(&parse_options, configuration);
+  configuration = cfg_new_snippet();
+  cfg_load_module(configuration, "disk-buffer");
+
   set_mark_message_serialized_size();
+}
 
-  test_over_EOF();
-
-  test_rewind_backlog();
-
+static void
+teardown(void)
+{
   cfg_free(configuration);
   app_shutdown();
-
-  return 0;
 }
+
+TestSuite(diskq_reliable, .init = setup, .fini = teardown);
