@@ -222,6 +222,12 @@ Test(format_json, test_v40_value_pairs_yields_typed_values)
   /* macro */
   assert_template_format("$(format-json --auto-cast FACILITY_NUM)",
                          "{\"FACILITY_NUM\":19}");
+
+  /* RFC8259 number sanitization */
+  assert_template_format("$(format-json num=int(00014))", "{\"num\":14}");
+  assert_template_format("$(format-json num=int(+14))", "{\"num\":14}");
+  assert_template_format("$(format-json num=int(-025))", "{\"num\":-25}");
+  assert_template_format("$(format-json num=int(+0009223372036854775804))", "{\"num\":9223372036854775804}");
 }
 
 Test(format_json, test_cast_option_always_yields_strings_regardless_of_versions)
@@ -335,11 +341,14 @@ Test(format_json, test_format_json_with_utf8)
   LogMessage *msg = create_empty_message();
   log_msg_set_value_by_name(msg, "UTF8-C2", "\xc2\xbf \xc2\xb6 \xc2\xa9 \xc2\xb1", -1); // ¿ ¶ © ±
   log_msg_set_value_by_name(msg, "UTF8-C3", "\xc3\x88 \xc3\x90", -1); // È Ð
+  log_msg_set_value_by_name(msg, "UTF8-CTRL", "\x07\x09", -1);
 
   assert_template_format_msg("$(format-json MSG=\"${UTF8-C2}\")", "{\"MSG\":\"\xc2\xbf \xc2\xb6 \xc2\xa9 \xc2\xb1\"}",
                              msg);
   assert_template_format_msg("$(format-json MSG=\"${UTF8-C3}\")", "{\"MSG\":\"\xc3\x88 \xc3\x90\"}", msg);
 
+  assert_template_format_msg("$(format-json MSG=\"${UTF8-CTRL}\")", "{\"MSG\":\"\\u0007\\t\"}", msg);
+
   log_msg_unref(msg);
 }
 
