@@ -26,6 +26,7 @@
 #define _SNG_PYTHON_DEST_H
 
 #include "python-module.h"
+#include "python-options.h"
 #include "driver.h"
 #include "logwriter.h"
 #include "value-pairs/value-pairs.h"
@@ -34,7 +35,7 @@ LogDriver *python_dd_new(GlobalConfig *cfg);
 void python_dd_set_loaders(LogDriver *d, GList *loaders);
 void python_dd_set_class(LogDriver *d, gchar *class_name);
 void python_dd_set_value_pairs(LogDriver *d, ValuePairs *vp);
-void python_dd_set_option(LogDriver  *d, gchar *key, gchar *value);
+void python_dd_set_options(LogDriver *d, PythonOptions *options);
 LogTemplateOptions *python_dd_get_template_options(LogDriver *d);
 
 void py_log_destination_global_init(void);
