@@ -31,6 +31,7 @@
 #include "python-persist.h"
 #include "python-ack-tracker.h"
 #include "python-bookmark.h"
+#include "python-flags.h"
 
 #include <structmember.h>
 
@@ -40,7 +41,7 @@ typedef struct _PythonFetcherDriver
 
   gchar *class;
   GList *loaders;
-  GHashTable *options;
+  PythonOptions *options;
 
   struct
   {
@@ -75,11 +76,12 @@ python_fetcher_set_class(LogDriver *s, gchar *filename)
 }
 
 void
-python_fetcher_set_option(LogDriver *s, gchar *key, gchar *value)
+python_fetcher_set_options(LogDriver *s, PythonOptions *options)
 {
   PythonFetcherDriver *self = (PythonFetcherDriver *) s;
-  gchar *normalized_key = __normalize_key(key);
-  g_hash_table_insert(self->options, normalized_key, g_strdup(value));
+
+  python_options_free(self->options);
+  self->options = options;
 }
 
 void
@@ -114,10 +116,10 @@ _pf_py_invoke_void_method_by_name(PythonFetcherDriver *self, const gchar *method
 }
 
 static gboolean
-_pf_py_invoke_bool_method_by_name_with_args(PythonFetcherDriver *self, const gchar *method_name)
+_pf_py_invoke_bool_method_by_name_with_options(PythonFetcherDriver *self, const gchar *method_name)
 {
-  return _py_invoke_bool_method_by_name_with_args(self->py.instance, method_name, self->options, self->class,
-                                                  self->super.super.super.super.id);
+  return _py_invoke_bool_method_by_name_with_options(self->py.instance, method_name, self->options, self->class,
+                                                     self->super.super.super.super.id);
 }
 
 static void
@@ -135,7 +137,7 @@ _pf_py_invoke_bool_function(PythonFetcherDriver *self, PyObject *func, PyObject
 static gboolean
 _py_invoke_init(PythonFetcherDriver *self)
 {
-  return _pf_py_invoke_bool_method_by_name_with_args(self, "init");
+  return _pf_py_invoke_bool_method_by_name_with_options(self, "init");
 }
 
 static void
@@ -454,6 +456,34 @@ _py_init_ack_tracker_factory(PythonFetcherDriver *self)
   return TRUE;
 }
 
+static gboolean
+_py_set_flags(PythonFetcherDriver *self)
+{
+  MsgFormatOptions *parse_options = log_threaded_source_driver_get_parse_options(&self->super.super.super.super);
+  g_assert(parse_options);
+
+  PyObject *flags = python_source_flags_new(parse_options->flags);
+  if (!flags)
+    return FALSE;
+
+  if (PyObject_SetAttrString(self->py.instance, "flags", flags) == -1)
+    {
+      gchar buf[256];
+
+      msg_error("python-fetcher: Error setting flags attribute",
+                evt_tag_str("driver", self->super.super.super.super.id),
+                evt_tag_str("class", self->class),
+                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+      _py_finish_exception_handling();
+
+      Py_DECREF(flags);
+      return FALSE;
+    }
+
+  Py_DECREF(flags);
+  return TRUE;
+}
+
 static gboolean
 _py_set_parse_options(PythonFetcherDriver *self)
 {
@@ -522,6 +552,9 @@ _py_fetcher_init(PythonFetcherDriver *self)
   if (!_py_init_bindings(self))
     goto error;
 
+  if (!_py_set_flags(self))
+    goto error;
+
   if (self->py.open_method)
     self->super.connect = python_fetcher_open;
 
@@ -637,7 +670,7 @@ python_fetcher_free(LogPipe *s)
   PyGILState_Release(gstate);
 
   g_free(self->class);
-  g_hash_table_unref(self->options);
+  python_options_free(self->options);
   string_list_free(self->loaders);
 
   log_threaded_fetcher_driver_free_method(s);
@@ -660,7 +693,7 @@ python_fetcher_new(GlobalConfig *cfg)
 
   self->super.fetch = python_fetcher_fetch;
 
-  self->options = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+  self->options = python_options_new();
 
   return &self->super.super.super.super;
 }
