@@ -25,12 +25,13 @@
 #define SNG_PYTHON_FETCHER_H
 
 #include "python-module.h"
+#include "python-options.h"
 #include "driver.h"
 
 LogDriver *python_fetcher_new(GlobalConfig *cfg);
 void python_fetcher_set_loaders(LogDriver *d, GList *loaders);
 void python_fetcher_set_class(LogDriver *d, gchar *class_name);
-void python_fetcher_set_option(LogDriver *d, gchar *key, gchar *value);
+void python_fetcher_set_options(LogDriver *d, PythonOptions *options);
 
 void py_log_fetcher_global_init(void);
 
