@@ -31,6 +31,7 @@
 #include "python-persist.h"
 #include "python-ack-tracker.h"
 #include "python-bookmark.h"
+#include "python-flags.h"
 
 #include <structmember.h>
 
@@ -42,7 +43,7 @@ struct _PythonSourceDriver
 
   gchar *class;
   GList *loaders;
-  GHashTable *options;
+  PythonOptions *options;
   ThreadId thread_id;
 
   void (*post_message)(PythonSourceDriver *self, LogMessage *msg);
@@ -82,11 +83,12 @@ python_sd_set_class(LogDriver *s, gchar *filename)
 }
 
 void
-python_sd_set_option(LogDriver *s, gchar *key, gchar *value)
+python_sd_set_options(LogDriver *s, PythonOptions *options)
 {
   PythonSourceDriver *self = (PythonSourceDriver *) s;
-  gchar *normalized_key = __normalize_key(key);
-  g_hash_table_insert(self->options, normalized_key, g_strdup(value));
+
+  python_options_free(self->options);
+  self->options = options;
 }
 
 void
@@ -121,10 +123,10 @@ _ps_py_invoke_void_method_by_name(PythonSourceDriver *self, const gchar *method_
 }
 
 static gboolean
-_ps_py_invoke_bool_method_by_name_with_args(PythonSourceDriver *self, const gchar *method_name)
+_ps_py_invoke_bool_method_by_name_with_options(PythonSourceDriver *self, const gchar *method_name)
 {
-  return _py_invoke_bool_method_by_name_with_args(self->py.instance, method_name, self->options, self->class,
-                                                  self->super.super.super.id);
+  return _py_invoke_bool_method_by_name_with_options(self->py.instance, method_name, self->options, self->class,
+                                                     self->super.super.super.id);
 }
 
 static void
@@ -136,7 +138,7 @@ _ps_py_invoke_void_function(PythonSourceDriver *self, PyObject *func, PyObject *
 static gboolean
 _py_invoke_init(PythonSourceDriver *self)
 {
-  return _ps_py_invoke_bool_method_by_name_with_args(self, "init");
+  return _ps_py_invoke_bool_method_by_name_with_options(self, "init");
 }
 
 static void
@@ -343,6 +345,39 @@ _py_init_bindings(PythonSourceDriver *self)
   return initialized;
 }
 
+static gboolean
+_py_set_flags(PythonSourceDriver *self)
+{
+  MsgFormatOptions *parse_options = log_threaded_source_driver_get_parse_options(&self->super.super.super);
+  g_assert(parse_options);
+
+  PyObject *flags = python_source_flags_new(parse_options->flags);
+  if (!flags)
+    {
+      msg_error("python-source: Error creating flags attribute",
+                evt_tag_str("driver", self->super.super.super.id),
+                evt_tag_str("class", self->class));
+      return FALSE;
+    }
+
+  if (PyObject_SetAttrString(self->py.instance, "flags", flags) == -1)
+    {
+      gchar buf[256];
+
+      msg_error("python-source: Error setting flags attribute",
+                evt_tag_str("driver", self->super.super.super.id),
+                evt_tag_str("class", self->class),
+                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+      _py_finish_exception_handling();
+
+      Py_DECREF(flags);
+      return FALSE;
+    }
+
+  Py_DECREF(flags);
+  return TRUE;
+}
+
 static gboolean
 _py_init_object(PythonSourceDriver *self)
 {
@@ -482,6 +517,9 @@ _py_sd_init(PythonSourceDriver *self)
   if (!_py_init_bindings(self))
     goto error;
 
+  if (!_py_set_flags(self))
+    goto error;
+
   if (!_py_init_object(self))
     goto error;
 
@@ -702,7 +740,7 @@ python_sd_free(LogPipe *s)
   PyGILState_Release(gstate);
 
   g_free(self->class);
-  g_hash_table_unref(self->options);
+  python_options_free(self->options);
   string_list_free(self->loaders);
 
   log_threaded_source_driver_free_method(s);
@@ -726,7 +764,7 @@ python_sd_new(GlobalConfig *cfg)
   self->super.request_exit = python_sd_request_exit;
   self->super.run = python_sd_run;
 
-  self->options = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+  self->options = python_options_new();
   self->post_message = _post_message_blocking;
 
   return &self->super.super.super;
