@@ -96,6 +96,14 @@ _load_code(const gchar *code)
 
 TestSuite(python_persist_name, .init = setup, .fini = teardown);
 
+static PythonOptions *
+_create_custom_options_with_dummy_option(void)
+{
+  PythonOptions *custom_options = python_options_new();
+  python_options_add_option(custom_options, python_option_string_new("key", "value"));
+  return custom_options;
+}
+
 const gchar *python_destination_code = "\n\
 from _syslogng import LogDestination\n\
 class Dest(LogDestination):\n\
@@ -111,7 +119,8 @@ Test(python_persist_name, test_python_dest)
 
   LogDriver *d = python_dd_new(empty_cfg);
   python_dd_set_class(d, "Dest");
-  python_dd_set_option(d, "key", "value");
+  python_dd_set_options(d, _create_custom_options_with_dummy_option());
+
   cr_assert(log_pipe_init((LogPipe *)d));
 
   cr_assert_str_eq(log_pipe_get_persist_name((LogPipe *)d), "python.value");
@@ -136,7 +145,8 @@ Test(python_persist_name, test_python_fetcher)
 
   LogDriver *d = python_fetcher_new(empty_cfg);
   python_fetcher_set_class(d, "Fetcher");
-  python_fetcher_set_option(d, "key", "value");
+  python_fetcher_set_options(d, _create_custom_options_with_dummy_option());
+
   cr_assert(log_pipe_init((LogPipe *)d));
 
   cr_assert_str_eq(log_pipe_get_persist_name((LogPipe *)d), "python-fetcher.value");
@@ -163,7 +173,8 @@ Test(python_persist_name, test_python_source)
 
   LogDriver *d = python_sd_new(empty_cfg);
   python_sd_set_class(d, "Source");
-  python_sd_set_option(d, "key", "value");
+  python_sd_set_options(d, _create_custom_options_with_dummy_option());
+
   cr_assert(log_pipe_init((LogPipe *)d));
 
   cr_assert_str_eq(log_pipe_get_persist_name((LogPipe *)d), "python-source.value");
@@ -231,7 +242,8 @@ Test(python_persist_name, test_python_fetcher_no_generate_persist_name)
 
   LogDriver *d = python_fetcher_new(empty_cfg);
   python_fetcher_set_class(d, "Fetcher");
-  python_fetcher_set_option(d, "key", "value");
+  python_fetcher_set_options(d, _create_custom_options_with_dummy_option());
+
   log_pipe_set_persist_name((LogPipe *)d, "test_persist_name");
   cr_assert(log_pipe_init((LogPipe *)d));
 
@@ -256,7 +268,8 @@ Test(python_persist_name, test_python_source_no_generate_persist_name)
 
   LogDriver *d = python_sd_new(empty_cfg);
   python_sd_set_class(d, "Source");
-  python_sd_set_option(d, "key", "value");
+  python_sd_set_options(d, _create_custom_options_with_dummy_option());
+
   log_pipe_set_persist_name((LogPipe *)d, "test_persist_name");
   cr_assert(log_pipe_init((LogPipe *)d));
 
@@ -340,7 +353,8 @@ Test(python_persist_name, test_python_fetcher_persist_preference)
   LogDriver *d = python_fetcher_new(empty_cfg);
   log_pipe_set_persist_name(&d->super, "test_persist_name");
   python_fetcher_set_class(d, "Fetcher");
-  python_fetcher_set_option(d, "key", "value");
+  python_fetcher_set_options(d, _create_custom_options_with_dummy_option());
+
   cr_assert(log_pipe_init((LogPipe *)d));
 
   cr_assert_str_eq(log_pipe_get_persist_name((LogPipe *)d), "python-fetcher.test_persist_name");
@@ -358,7 +372,8 @@ Test(python_persist_name, test_python_source_persist_preference)
   LogDriver *d = python_sd_new(empty_cfg);
   log_pipe_set_persist_name(&d->super, "test_persist_name");
   python_sd_set_class(d, "Source");
-  python_sd_set_option(d, "key", "value");
+  python_sd_set_options(d, _create_custom_options_with_dummy_option());
+
   cr_assert(log_pipe_init((LogPipe *)d));
 
   cr_assert_str_eq(log_pipe_get_persist_name((LogPipe *)d), "python-source.test_persist_name");
